/*

alpha.c - Mission Alphatron

*/

#define	USE_MMTIMER
/*...sincludes:0:*/
#define	STRICT
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
/*...e*/
/*...svars:0:*/
static char appName [] = "Mission Alphatron";

static HANDLE hinst;			/* Our instance number               */
static HCURSOR hHourGlass;		/* Handle to hourglass cursor        */

#ifdef USE_MMTIMER
#define	T_TITLE      10
#define	T_CYCLE_INIT 12
#define	T_CYCLE_MIN   1
static DWORD dwTimeLast;
static DWORD dwTimeCycle;
#endif
/*...e*/
/*...sdebug:0:*/
static HWND hwndError;

static void debug(LPSTR message)
	{
	MessageBox(hwndError, message, "DEBUG", MB_OK | MB_ICONQUESTION);
	}

static BOOL debug_q(LPSTR message)
	{
	return ( MessageBox(hwndError, message, "DEBUG", MB_YESNO | MB_ICONQUESTION) == IDYES );
	}
/*...e*/
/*...sRandom:0:*/
#define	Random() ((unsigned int) rand())
#ifdef NEVER
static unsigned int Random(void)
	{
	static long last = 0x1287755;
	last = (last * last) ^ (last+1);
	return (unsigned int) ( last & 0x7fff );
	}
#endif
/*...e*/
/*...sThreeDeeRect:0:*/
static void ThreeDeeRect(
	HDC hdc,
	int x1, int y1, int x2, int y2,
	LONG rgbTopLeft, LONG rgb, LONG rgbBottomRight
	)
	{
	HPEN hpen = SelectObject(hdc, CreatePen(PS_SOLID,1, rgbTopLeft));
	if ( rgb != -1L )
		{
		HBRUSH hbrush = SelectObject(hdc, CreateSolidBrush(rgb));
		Rectangle(hdc, x1,y1,x2,y2);
		DeleteObject(SelectObject(hdc, hbrush));
		}
	else
		{
		MoveToEx(hdc, x1, y2-1, NULL);
		LineTo(hdc, x1, y1);
		LineTo(hdc, x2, y1);
		}
	DeleteObject(SelectObject(hdc, CreatePen(PS_SOLID, 1, rgbBottomRight)));
	MoveToEx(hdc, x2-1, y1, NULL);
	LineTo(hdc, x2-1, y2-1);
	LineTo(hdc, x1, y2-1);
	DeleteObject(SelectObject(hdc, hpen));
	}
/*...e*/
/*...sline ends:0:*/
/*
LineTo() draws upto, but not including the position you give it.
We wish to draw the last pixel sometimes, so we draw a pixel too far.
Where we do this, we use LINEEND to make this explicit.
*/

#define	LINEEND 1
/*...e*/
/*...swindow geometry:0:*/
#define	CX_EXTRA	((32+6)*2)
#define	CX_WINDOW	600
#define CX_ACTION	(CX_WINDOW+CX_EXTRA)

#define	CY_WINDOW	400
#define	CY_STATUS	 20
#define	CY_ACTION	(CY_WINDOW-CY_STATUS)

#define	CX_ANI 100
#define	CY_ANI 100
#define X_ANI (CX_WINDOW-20-CX_ANI)
#define	Y_ANI 20

#define	CX_TITLE 378
#define	CY_TITLE 182
#define	X_TITLE 20
#define	Y_TITLE 20
/*...e*/
/*...stitle timings:0:*/
#ifdef USE_PEEKMSG
#define T_IDLE 500000
#endif
#ifdef USE_MMTIMER
#define	T_IDLE 1000
#endif
#ifdef USE_WM_TIMER
#define	T_IDLE 1000
#endif

#define	T_ANI4 (30*4)
#define	T_ANI3 (30*3)
#define	T_ANI2 (30*2)
#define	T_ANI  30
/*...e*/
/*...sland sections:0:*/
/*
The landscape is divided into a top and bottom half.
Each is made of a set of land sections.
We keep info for the screen width CX_ACTION plus an extra bit off of the
right edge of the screen CX_EXTRA.
*/

typedef BYTE LSTYPE; /* lst */
#define	LST_FLAT	((LSTYPE)  1)
#define	LST_VERT_UP	((LSTYPE)  2)
#define	LST_VERT_DOWN	((LSTYPE)  4)
#define	LST_DIAG_UP	((LSTYPE)  8)
#define	LST_DIAG_DOWN	((LSTYPE) 16)

typedef struct { LSTYPE lst; int len; LONG rgb, rgbEdge; } LSECT; /* lsect */

#define	N_LSECT 150
static int yStartTop;
static int clsectTop;
static LSECT lsectTop[N_LSECT];
static int yStartBot;
static int clsectBot;
static LSECT lsectBot[N_LSECT];

/*...sBuildFlatTop:0:*/
static void BuildFlatTop(LONG rgb, LONG rgbEdge, int y)
	{
	yStartTop = y;
	clsectTop = 1;
	lsectTop[0].lst     = LST_FLAT;
	lsectTop[0].len     = CX_ACTION+CX_EXTRA;
	lsectTop[0].rgb     = rgb;
	lsectTop[0].rgbEdge = rgbEdge;
	}
/*...e*/
/*...sBuildFlatBot:0:*/
static void BuildFlatBot(LONG rgb, LONG rgbEdge, int yStart)
	{
	yStartBot = yStart;
	clsectBot = 1;
	lsectBot[0].lst     = LST_FLAT;
	lsectBot[0].len     = CX_ACTION+CX_EXTRA;
	lsectBot[0].rgb     = rgb;
	lsectBot[0].rgbEdge = rgbEdge;
	}
/*...e*/
/*...e*/
/*...sland as y\61\f\40\x\41\:0:*/
/*
As well as the 'land section' representation of the land, we will keep an
expanded version which has a height for each x.
*/

typedef struct { int y, cy; LONG rgb, rgbEdge; } YOX;

static YOX yoxTop[CX_ACTION+CX_EXTRA]; int yEndTop;
static YOX yoxBot[CX_ACTION+CX_EXTRA]; int yEndBot;

/*...sExpandTopYs:0:*/
static int ExpandTopYs(int clsect, LSECT lsect[], YOX yox[], int yVal)
	{
	int i, j;
	for ( i = 0; i < clsect; i++ )
		{
		LONG rgb = lsect[i].rgb, rgbEdge = lsect[i].rgbEdge;
		switch ( lsect[i].lst )
			{
			case LST_FLAT:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = yVal;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			case LST_VERT_UP:
				yox->y       = yVal;
				yox->cy      = lsect[i].len;
				yox->rgb     = rgb;
				yox->rgbEdge = rgbEdge;
				yox++;
				yVal -= lsect[i].len;
				break;
			case LST_VERT_DOWN:
				yVal += lsect[i].len;
				yox->y       = yVal;
				yox->cy      = lsect[i].len;
				yox->rgb     = rgb;
				yox->rgbEdge = rgbEdge;
				yox++;
				break;
			case LST_DIAG_UP:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = yVal--;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			case LST_DIAG_DOWN:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = ++yVal;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			}
		}
	return yVal;
	}
/*...e*/
/*...sExpandBotYs:0:*/
static int ExpandBotYs(int clsect, LSECT lsect[], YOX yox[], int yVal)
	{
	int i, j;
	for ( i = 0; i < clsect; i++ )
		{
		LONG rgb = lsect[i].rgb, rgbEdge = lsect[i].rgbEdge;
		switch ( lsect[i].lst )
			{
			case LST_FLAT:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = yVal;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			case LST_VERT_UP:
				yVal -= lsect[i].len;
				yox->y       = yVal;
				yox->cy      = 1;
				yox->rgb     = rgb;
				yox->rgbEdge = rgbEdge;
				yox++;
				break;
			case LST_VERT_DOWN:
				yox->y       = yVal;
				yox->cy      = 1;
				yox->rgb     = rgb;
				yox->rgbEdge = rgbEdge;
				yox++;
				yVal += lsect[i].len;
				break;
			case LST_DIAG_UP:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = --yVal;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			case LST_DIAG_DOWN:
				for ( j = 0; j < lsect[i].len; j++, yox++ )
					{
					yox->y       = yVal++;
					yox->cy      = 1;
					yox->rgb     = rgb;
					yox->rgbEdge = rgbEdge;
					}
				break;
			}
		}
	return yVal;
	}
/*...e*/
/*...sMaxTopY \45\ used for collision detection with top landscape:0:*/
static int MaxTopY(int x, int cx)
	{
	int i, y = 0; /* Smaller than any legal value */
	if ( x < 0 )
		{ cx += x; x = 0; }
	for ( i = 0; i < cx; i++ )
		if ( yoxTop[x+i].y > y )
			y = yoxTop[x+i].y;
	return y;
	}
/*...e*/
/*...sMinBotY \45\ used for collision detection with bottom landscape:0:*/
static int MinBotY(int x, int cx)
	{
	int i, y = CY_ACTION*2; /* Bigger than any legal value */
	if ( x < 0 )
		{ cx += x; x = 0; }
	for ( i = 0; i < cx; i++ )
		if ( yoxBot[x+i].y < y )
			y = yoxBot[x+i].y;
	return y;
	}
/*...e*/
/*...sHitLand \45\ used for collision detection with both:0:*/
static BOOL HitLand(int x, int y, int cx, int cy)
	{		
	return MaxTopY(x, cx) > y || MinBotY(x, cx) < y + cy;
	}
/*...e*/
/*...sLandscape \45\ draw landscape that overlaps the given rectangle\46\:0:*/
static void Landscape(HDC hdc, int x1, int y1, int x2, int y2, int cxScrollMax)
	{
	int x;
	YOX *yox = &yoxTop[x2-1+CX_EXTRA/2];

	LONG rgb      = yox->rgb;
	HPEN hpen     = CreatePen(PS_SOLID, 1, rgb);
	LONG rgbEdge  = yox->rgbEdge;
	HPEN hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);

	if ( x1 < x2-cxScrollMax )
		x1 = x2-cxScrollMax;

	for ( x = x2-1; x >= x1; x--, yox-- )
		if ( y1 < yox->y )
/*...sdraw top overlapping area:24:*/
{
int y = yox->y-1-yox->cy;
HPEN hpenOld;
if ( yox->rgb != rgb )
	{
	rgb = yox->rgb;
	DeleteObject(hpen);
	hpen = CreatePen(PS_SOLID, 1, rgb);
	}
if ( yox->rgbEdge != rgbEdge )
	{
	rgbEdge = yox->rgbEdge;
	DeleteObject(hpenEdge);
	hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);
	}
hpenOld = SelectObject(hdc, hpenEdge);
MoveToEx(hdc, x,yox->y-1, NULL);
LineTo(hdc, x,y);
if ( y1 <= y )
	{
	SelectObject(hdc, hpen);
	LineTo(hdc, x,y1-LINEEND);
	}
SelectObject(hdc, hpenOld);
}
/*...e*/

	yox = &yoxBot[x2-1+CX_EXTRA/2];

	for ( x = x2-1; x >= x1; x--, yox-- )
		if ( y2 > yox->y )
/*...sdraw bot overlapping area:24:*/
{
int y = yox->y+yox->cy;
HPEN hpenOld;
if ( yox->rgb != rgb )
	{
	rgb = yox->rgb;
	DeleteObject(hpen);
	hpen = CreatePen(PS_SOLID, 1, rgb);
	}
if ( yox->rgbEdge != rgbEdge )
	{
	rgbEdge = yox->rgbEdge;
	DeleteObject(hpenEdge);
	hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);
	}
hpenOld  = SelectObject(hdc, hpenEdge);
MoveToEx(hdc, x,yox->y, NULL);
LineTo(hdc, x,y);
if ( y < y2 )
	{
	SelectObject(hdc, hpen);
	LineTo(hdc, x,y2);
	}
SelectObject(hdc, hpenOld);
}
/*...e*/

	DeleteObject(hpen);
	DeleteObject(hpenEdge);
	}
/*...e*/
/*...e*/
/*...slevels:0:*/
static LSTYPE probConvert[5] =
	{ LST_FLAT, LST_VERT_UP, LST_VERT_DOWN, LST_DIAG_UP, LST_DIAG_DOWN };

typedef struct
	{
	int probTop[5];
	int probBot[5];
	int iMinLenFlat, iMaxLenFlat;
	int iMinLenVert, iMaxLenVert;
	int iMinLenDiag, iMaxLenDiag;
	int iMinDistApart, iMaxDistApart;
	int iFreqChange;
	LONG rgb, rgbEdge;
	} LEVEL;

#define	N_LEVELS 11

static LEVEL levels[N_LEVELS] =
	{
/* 0: Rockets */
		{
		{ 100,0,0,0,0 },
		{ 50,0,0,25,25 },
		100,120, -1,-1, 20,50, 120,400,
		10,
		RGB(255,0,0),RGB(255,255,255),
		},

/* 1: Missiles */
		{
		{ 50,0,0,25,25 },
		{ 70,0,0,15,15 },
		70,120, -1,-1, 40,150, 120,400,
		10,
		RGB(0,255,0),RGB(255,255,255),
		},

/* 2: Up and downers */
		{
		{ 40,30,30,0,0 },
		{ 40,30,30,0,0 },
		50,100, 50,100, -1,-1, 100,400,
		10,
		RGB(0,0,255),RGB(0,255,255),
		},

/* 3: Left flying ships */
		{
		{ 60,20,20,0,0 },
		{ 60,20,20,0,0 },
		50,80, 70,80, 20,50, 120,400,
		10,
		RGB(0,128,0),RGB(255,255,255),
		},

/* 4: Acid droppers */
		{
		{ 50,25,25,0,0 },
		{ 50,25,25,0,0 },
		50,80, 30,70, 20,50, 100,250,
		10,
		RGB(128,128,0),RGB(255,255,255),
		},

/* 5: Bouncers */
		{
		{ 20,20,20,20,20 },
		{ 40,15,15,15,15 },
		60,100, 50,100, 50,100, 120,400,
		10,
		RGB(0,0,255),RGB(255,255,255),
		},

/* 6: Shuttles and Asteroids */
		{
		{ 80,10,10,0,0 },
		{ 70,15,15,0,0 },
		60,100, 50,100, -1,-1, 120,400,
		15,
		RGB(0,255,255),RGB(0,0,255),
		},

/* 7: RotorDs and Rotors */
		{
		{ 70,0,0,10,20 },
		{ 60,0,0,30,10 }, /* Tends to close the gap */
		60,100, -1,-1, 50,100, 120,400,
		10,
		RGB(0,0,128),RGB(0,0,255),
		},

/* 8: Cute guys */
		{
		{ 60,0,0,20,20 },
		{ 60,0,0,20,20 },
		60,100, -1,-1, 50,100, 120,400,
		10,
		RGB(128,0,0), RGB(255,255,255),
		},

/* 9: A mixture (diagonal land) */
		{
		{ 10,15,15,30,30 },
		{ 80,0,0,10,10 },
		100,150, 70,100, 30,70, 120,300,
		12,
		RGB(128,0,128),RGB(255,255,255),
		},

/* 10: A mixture (square land) */
		{
		{ 40,30,30,0,0 },
		{ 40,30,30,0,0 },
		100,150, 100,200, -1,-1, 120,300,
		12,
		RGB(0,0,0),RGB(255,255,255),
		},
	};

static int iLevel = 0;
static LEVEL *level = &levels[0];

static void SetLevel(int iLevelNew)
	{
	iLevel = iLevelNew;
	level = &levels[iLevel];
	}

static void NextLevel(void)
	{
	if ( iLevel == N_LEVELS-1 )
		SetLevel(1);
	else
		SetLevel(iLevel+1);
#ifdef USE_MMTIMER
	if ( dwTimeCycle > T_CYCLE_MIN )
		--dwTimeCycle;
#endif
	}
/*...e*/
/*...sBuildInitLand:0:*/
static void BuildInitLand(void)
	{
	BuildFlatTop(level->rgb, level->rgbEdge, iLevel == 0 ? 10 : 20 );
	BuildFlatBot(level->rgb, level->rgbEdge, CY_ACTION-20);
	yEndTop = ExpandTopYs(clsectTop, lsectTop, yoxTop, yStartTop);
	yEndBot = ExpandBotYs(clsectBot, lsectBot, yoxBot, yStartBot);
	}
/*...e*/
/*...ssprite resources:0:*/
typedef struct { int cx,cy; char *szResName; HICON hicon; } SPRITE;

#define SPR_SHIP     0
#define SPR_FUEL     1
#define SPR_FUEL2    2
#define SPR_ARIEL    3
#define SPR_NUKE     4
#define SPR_ROCKET1  5
#define SPR_ROCKET2  6
#define SPR_MISSILEG 7
#define SPR_MISSILEF 8
#define SPR_MISSILEU 9
#define SPR_MISSILED 10
#define	SPR_UD1      11
#define	SPR_UD2      12
#define	SPR_UD3      13
#define	SPR_UD4      14
#define	SPR_MINE1    15
#define	SPR_MINE2    16
#define	SPR_LSHIP    17
#define	SPR_LSHIPFR  18
#define	SPR_LSHIPFB  19
#define	SPR_ACIDSHIP 20
#define	SPR_ACID0    21
#define	SPR_ACID1    22
#define	SPR_BOUNCE1  23
#define	SPR_BOUNCE2  24
#define	SPR_BOUNCE3  25
#define	SPR_SHUT0    26
#define	SPR_SHUT1    27
#define	SPR_SHUT2    28
#define	SPR_SHUT3    29
#define	SPR_ASTER0   30
#define	SPR_ASTER1   31
#define	SPR_ASTER2   32
#define	SPR_ASTER3   33
#define	SPR_ASTER4   34
#define	SPR_ASTER5   35
#define	SPR_ASTER6   36
#define	SPR_ROTOR0   37
#define	SPR_ROTOR1   38
#define	SPR_ROTORD0  39
#define	SPR_ROTORD1  40
#define	SPR_CUTE0    41
#define	SPR_CUTE1    42
#define	SPR_CUTE2    43
#define	SPR_BON100A  44
#define	SPR_BON100B  45
#define	SPR_BON500A  46
#define	SPR_BON500B  47
#define	SPR_BON1000A 48
#define	SPR_BON1000B 49
#define	SPR_BON1000C 50
#define	SPR_32X32E   51
#define	SPR_32X15E   56
#define	SPR_NEXT     61

static SPRITE sprites[] = 
	{
	  { 32,27, "IconShip"     },
	  { 32,28, "IconFuel"     },
	  { 32,32, "IconFuel2"    },
	  { 32,32, "IconAriel"    },
	  { 32,27, "IconNuke"     },
	  { 32,32, "IconRocket1"  },
	  { 32,32, "IconRocket2"  },
	  { 32,18, "IconMissileG" },
	  { 32,15, "IconMissileF" },
	  { 32,16, "IconMissileU" },
	  { 32,16, "IconMissileD" },
	  { 24,32, "IconUd1"      },
	  { 24,32, "IconUd2"      },
	  { 24,32, "IconUd3"      },
	  { 24,32, "IconUd4"      },
	  {  6, 6, "IconMine1"    },
	  {  6, 6, "IconMine2"    },
	  { 32,21, "IconLship"    },
	  { 32,21, "IconLshipFr"  },
	  { 32,21, "IconLshipFb"  },
	  { 32,28, "IconAcidShip" },
	  { 32,32, "IconAcid0"    },
	  { 32,32, "IconAcid1"    },
	  { 32,32, "IconBounce1"  },
	  { 32,32, "IconBounce2"  },
	  { 32,32, "IconBounce3"  },
	  { 32,32, "IconShut0"    },
	  { 32,32, "IconShut1"    },
	  { 32,32, "IconShut2"    },
	  { 32,32, "IconShut3"    },
	  { 32,32, "IconAster0"   },
	  { 32,32, "IconAster1"   },
	  { 32,32, "IconAster2"   },
	  { 32,32, "IconAster3"   },
	  { 32,32, "IconAster4"   },
	  { 32,32, "IconAster5"   },
	  { 32,32, "IconAster6"   },
	  { 32,32, "IconRotor0"   },
	  { 32,32, "IconRotor1"   },
	  { 32,32, "IconRotorD0"  },
	  { 32,32, "IconRotorD1"  },
	  { 32,32, "IconCute0"    },
	  { 32,32, "IconCute1"    },
	  { 32,32, "IconCute2"    },
	  { 32,32, "IconBon100a"  },
	  { 32,32, "IconBon100b"  },
	  { 32,32, "IconBon500a"  },
	  { 32,32, "IconBon500b"  },
	  { 32,32, "IconBon1000a" },
	  { 32,32, "IconBon1000b" },
	  { 32,32, "IconBon1000c" },
	  { 32,32, "Icon32x32e1"  },
	  { 32,32, "Icon32x32e2"  },
	  { 32,32, "Icon32x32e3"  },
	  { 32,32, "Icon32x32e4"  },
	  { 32,32, "Icon32x32e5"  },
	  { 32,15, "Icon32x15e1"  },
	  { 32,15, "Icon32x15e2"  },
	  { 32,15, "Icon32x15e3"  },
	  { 32,15, "Icon32x15e4"  },
	  { 32,15, "Icon32x15e5"  },
	};

#define	N_SPRITES (sizeof(sprites)/sizeof(sprites[0]))

/*...sLoadSprites:0:*/
static void LoadSprites(void)
	{
	int i;
	for ( i = 0; i < N_SPRITES; i++ )
		sprites[i].hicon = LoadIcon(hinst, sprites[i].szResName);
	}
/*...e*/
/*...sFreeSprites:0:*/
static void FreeSprites(void)
	{
	int i;
	for ( i = 0; i < N_SPRITES; i++ )
		DestroyIcon(sprites[i].hicon);
	}
/*...e*/
/*...e*/
/*...sglobals:0:*/
static HBITMAP hbmDigits;
static HBITMAP hbmScore;
static HBITMAP hbmHiScore;
static HBITMAP hbmFuel;
static HBITMAP hbmLifeSym;

static int iCycle;

static BOOL bUpPressed    = FALSE;
static BOOL bDownPressed  = FALSE;
static BOOL bLeftPressed  = FALSE;
static BOOL bRightPressed = FALSE;
static BOOL bSpacePressed = FALSE;

#define	MAX_FUEL 308
static int iFuel, iFuelLast;

static int iScore = 0, iHiScore = 0;

#define	RGB_FUELBAR RGB(0,255,0)

static int xShip, yShip;

static int iLaserHeat;
static BOOL bLaserOn;
static int xLaser, yLaser, cxLaser;

static BOOL bDead;
static int iDeathState; /* Only valid when bDead */
static int iLives;

#define	MODE_INIT  0
#define	MODE_TITLE 1
#define	MODE_GAME  2
#define MODE_DEMO  3
static int iMode = MODE_INIT;

static int iScrollBy = 2;
static BOOL bKeepGoing = TRUE;

#define	CX_SHIP 32
#define	CY_SHIP 27
#define	X_SHIP_INIT (CX_EXTRA/2+50)
#define Y_SHIP_INIT (CY_ACTION/2)

static long lCounter = 0;
/*...e*/
/*...sgeometry of ship sprite:0:*/
#define	 X_HIT_SHIP1  0
#define	 Y_HIT_SHIP1  0
#define CX_HIT_SHIP1 15
#define CY_HIT_SHIP1 27
#define  X_HIT_SHIP2 15
#define  Y_HIT_SHIP2  6
#define CX_HIT_SHIP2 16
#define CY_HIT_SHIP2 15
/*...e*/
/*...sPaintScore:0:*/
static void PaintScore(HDC hdc, int x, int y, int iScore)
	{
	HDC hdcMem = CreateCompatibleDC(hdc);
	HBITMAP hbmOld = SelectObject(hdcMem, hbmDigits);
	char szScore[6+1];
	int i;

	/* Write the digits of the score */
	sprintf(szScore, "%05d0", iScore);
	for ( i = 0; i < 6; i++ )
		BitBlt(hdc, x+i*12, y, 10, 12,
		       hdcMem, (szScore[i]-'0')*10, 0, SRCCOPY);

	SelectObject(hdcMem, hbmOld);
	DeleteDC(hdcMem);
	}
/*...e*/
/*...sPaintLives:0:*/
/*
Lives can be 3,2 or 1 usually.
It can go as high as 4 or 5 if score high enough.
We wish to display lives-1 symbols.
Therefore upto 4 symbols to be displayed.
*/

static void PaintLives(HDC hdc, int x, int y, int iLives)
	{
	HDC hdcMem = CreateCompatibleDC(hdc);
	HBITMAP hbmOld = SelectObject(hdcMem, hbmLifeSym);
	int i;

	for ( i = 0; i < 4; i++ )
		BitBlt(hdc, x+i*16, y+1, 16, 10,
		       hdcMem, (i+1<iLives)?0:16, 0, SRCCOPY);

	SelectObject(hdcMem, hbmOld);
	DeleteDC(hdcMem);
	}
/*...e*/
/*...sPaintFuelBar:0:*/
static void PaintFuelBar(HDC hdc, int x, int y, int iFuel)
	{
	if ( iFuel > iFuelLast )
		{
		if ( iFuelLast > 0 )
/*...sextend length of fuel bar:24:*/
{
HPEN hpenTopLeft     = CreatePen(PS_SOLID, 1, RGB(255,255,255));
HPEN hpenBottomRight = CreatePen(PS_SOLID, 1, RGB(128,128,128));
HPEN hpen            = CreatePen(PS_SOLID, 1, RGB_FUELBAR);
HBRUSH hbrush        = CreateSolidBrush(RGB_FUELBAR);
HPEN hpenOld         = SelectObject(hdc, hpen);
HBRUSH hbrushOld     = SelectObject(hdc, hbrush);
Rectangle(hdc, x+iFuelLast-1, y+2+2+1, x+iFuel-1, y+CY_STATUS-2-2-1);
SelectObject(hdc, hpenTopLeft);
MoveToEx(hdc, x+iFuelLast-1, y+2+2, NULL);
LineTo(hdc, x+iFuel-1, y+2+2);
SelectObject(hdc, hpenBottomRight);
LineTo(hdc, x+iFuel-1, y+CY_STATUS-2-2-1);
LineTo(hdc, x+iFuelLast-1-LINEEND, y+CY_STATUS-2-2-1);
SelectObject(hdc, hpenOld);
SelectObject(hdc, hbrushOld);
DeleteObject(hpenTopLeft);
DeleteObject(hpenBottomRight);
DeleteObject(hpen);
DeleteObject(hbrush);
}
/*...e*/
		else
/*...sextend length of fuel bar from zero:24:*/
ThreeDeeRect(hdc, x, y+2+2, x+iFuel, y+CY_STATUS-2-2,
	RGB(255,255,255), RGB_FUELBAR, RGB(192,192,192));
/*...e*/
		}
	else if ( iFuel < iFuelLast )
		{
		if ( iFuel > 0 )
/*...sdecrease length of fuel bar:24:*/
{
HPEN hpenBottomRight = CreatePen(PS_SOLID, 1, RGB(128,128,128));
HPEN hpen            = CreatePen(PS_SOLID, 1, RGB(192,192,192));
HBRUSH hbrush        = CreateSolidBrush(RGB(192,192,192));
HPEN hpenOld         = SelectObject(hdc, hpen);
HBRUSH hbrushOld     = SelectObject(hdc, hbrush);
Rectangle(hdc, x+iFuel, y+2+2, x+iFuelLast, y+CY_STATUS-2-2);
SelectObject(hdc, hpenBottomRight);
MoveToEx(hdc, x+iFuel-1, y+2+2, NULL);
LineTo(hdc, x+iFuel-1, y+CY_STATUS-2-2-1+LINEEND);
SelectObject(hdc, hpenOld);
SelectObject(hdc, hbrushOld);
DeleteObject(hpenBottomRight);
DeleteObject(hpen);
DeleteObject(hbrush);
}
/*...e*/
		else
/*...sdecrease length of fuel bar to zero:24:*/
{
HPEN hpen            = CreatePen(PS_SOLID, 1, RGB(192,192,192));
HBRUSH hbrush        = CreateSolidBrush(RGB(192,192,192));
HPEN hpenOld         = SelectObject(hdc, hpen);
HBRUSH hbrushOld     = SelectObject(hdc, hbrush);
Rectangle(hdc, x+iFuel, y+2+2, x+iFuelLast, y+CY_STATUS-2-2);
SelectObject(hdc, hpenOld);
SelectObject(hdc, hbrushOld);
DeleteObject(hpen);
DeleteObject(hbrush);
}
/*...e*/
		}
	iFuelLast = iFuel;
	}
/*...e*/
/*...sPaintLoadedBitmap:0:*/
static void PaintLoadedBitmap(
	HDC hdc, HBITMAP hbm,
	int x, int y, int cx, int cy
	)
	{
	HDC hdcMem = CreateCompatibleDC(hdc);
	HBITMAP hbmOld = SelectObject(hdcMem, hbm);
	BitBlt(hdc, x, y, cx, cy, hdcMem, 0, 0, SRCCOPY);
	SelectObject(hdcMem, hbmOld);
	DeleteDC(hdcMem);
	}
/*...e*/
/*...sPaintBitmap:0:*/
static void PaintBitmap(
	HDC hdc, char *szResName,
	int x, int y, int cx, int cy
	)
	{
	HBITMAP hbm = LoadBitmap(hinst, szResName);
	PaintLoadedBitmap(hdc, hbm, x, y, cx, cy);
	DeleteObject(hbm);
	}
/*...e*/
/*...sPaintAniFrame:0:*/
static void PaintAniFrame(HDC hdc, int iFrame)
	{
	char szResName[30+1];
	sprintf(szResName, "BitmapToff%03d", iFrame);
	PaintBitmap(hdc, szResName, X_ANI, Y_ANI, CX_ANI, CY_ANI);
	}
/*...e*/
/*...ssprites etc\46\:0:*/
/*...scomment on how this stuff works \40\will work actually\41\:0:*/
/*

plot all sprites, keeping list of { pos, size } plotted as 'erase-list'
loop
	move landscape model left by scroll amount
	move everything left by scroll amount
	move bad guys etc. however they move
	compute list of sprites to draw as { pos, size, bits } as 'draw-list'
	erase-list -= draw-list
	foreach erase-list element
		clear that area
		plot landscape
	foreach draw-list element
		clear that area
	foreach draw-list element
		'or' draw the sprite
	scrolldraw the landscape
	foreach erase-list + draw-list element
		blit to screen
	erase-list = draw-list

*/
/*...e*/

typedef struct _AREA
	{
	int x, y, cx, cy, cxScrollMax;
	HICON hicon;
	struct _AREA *areaNext;
	} AREA;

/*...sMallocArea\44\ FreeAreas \45\ memory allocation:0:*/
static AREA *areaFree = NULL;

static AREA *MallocArea(void)
	{
	AREA *area;
	if ( areaFree == NULL )
		return malloc(sizeof(AREA));
	area = areaFree;
	areaFree = areaFree->areaNext;
	return area;
	}

static AREA *ConstructArea(
	int x, int y, int cx, int cy,
	int cxScrollMax,
	HICON hicon
	)
	{
	AREA *area;
	if ( (area = MallocArea()) == NULL )
		return NULL;
	area->x = x;
	area->y = y;
	area->cx = cx;
	area->cy = cy;
	area->cxScrollMax = cxScrollMax;
	area->hicon = hicon;
	return area;
	}

static void FreeAreas(AREA *area)
	{
	AREA *areaLast;
	if ( area == NULL )
		return;
	for ( areaLast = area;
	      areaLast->areaNext != NULL;
	      areaLast = areaLast->areaNext )
		;
	areaLast->areaNext = areaFree;
	areaFree = area;
	}	
/*...e*/
/*...sEraseAreas            \45\ clear all areas in list to black:0:*/
static void EraseAreas(AREA *area, HDC hdc)
	{
	LONG rgbBlack    = RGB(0,0,0);
	HPEN hpenOld     = SelectObject(hdc, CreatePen(PS_SOLID, 1, rgbBlack));
	HBRUSH hbrushOld = SelectObject(hdc, CreateSolidBrush(rgbBlack));
	for ( ; area != NULL; area = area->areaNext )
		Rectangle(hdc, area->x,area->y,area->x+area->cx,area->y+area->cy);
	DeleteObject(SelectObject(hdc, hpenOld));
	DeleteObject(SelectObject(hdc, hbrushOld));
	}		
/*...e*/
/*...sLandscapeAreas        \45\ add landscape to areas \40\where applicable\41\:0:*/
/*
Assertions:
Mostly, the sprites will not overlap the landscape by very much, if at all.
*/

static void LandscapeAreas(AREA *area, HDC hdc)
	{
	for ( ; area != NULL; area = area->areaNext )
		Landscape(hdc,area->x,area->y,area->x+area->cx,area->y+area->cy,area->cxScrollMax);
	}		
/*...e*/
/*...sPlotAreas             \45\ plot sprites into areas of a DC:0:*/
static void PlotAreas(AREA *area, HDC hdc)
	{
	for ( ; area != NULL; area = area->areaNext )
		if ( area->hicon == NULL )
			{
			HPEN hpenOld = SelectObject(hdc, CreatePen(PS_SOLID, 1, RGB(0,255,255)));
			MoveToEx(hdc,area->x,area->y, NULL);
			LineTo(hdc,area->x+area->cx,area->y);
			DeleteObject(SelectObject(hdc, hpenOld));
			}
		else
			DrawIcon(hdc, area->x, area->y, area->hicon);
	}
/*...e*/
/*...sRefreshAreas          \45\ transfer all areas from one DC to another:0:*/
static void RefreshAreas(AREA *area, HDC hdcSrc, HDC hdcDest)
	{
	for ( ; area != NULL; area = area->areaNext )
		BitBlt(hdcDest, area->x, area->y, area->cx, area->cy,
		       hdcSrc, area->x, area->y, SRCCOPY);
	}		
/*...e*/

static AREA *areaErase = NULL;	/* Areas to erase next time around */
static AREA *areaDraw = NULL;	/* Areas to draw with sprite info */

static HDC hdcBacking;
static HBITMAP hbmBacking, hbmBackingOld;

/*...sResetAreas            \45\ reset everything back to initial state:0:*/
static void ResetAreas(void)
	{
	FreeAreas(areaDraw ); areaDraw = NULL;
	FreeAreas(areaErase); areaErase = NULL;
	}
/*...e*/
/*...e*/
/*...sScrollLeft:0:*/
/*
Extend the landscape to the right.
Use the level table entry for the current level.
*/

/*...sConsumeLeft:0:*/
static void ConsumeLeft(
	int *clsect, LSECT lsect[],
	int *yStart, YOX yox[],
	int xScroll
	)
	{
	int lsi = 0, x = xScroll;
	while ( x > 0 )
		switch ( lsect[lsi].lst )
			{
/*...sLST_FLAT:24:*/
case LST_FLAT:
	if ( x < lsect[lsi].len )
		{
		lsect[lsi].len -= x;
		x = 0;
		}
	else
		{
		x -= lsect[lsi].len;
		lsi++;
		}
	break;
/*...e*/
/*...sLST_VERT_UP:24:*/
case LST_VERT_UP:
	(*yStart) -= lsect[lsi].len;
	lsi++;
	x--;
	break;
/*...e*/
/*...sLST_VERT_DOWN:24:*/
case LST_VERT_DOWN:
	(*yStart) += lsect[lsi].len;
	lsi++;
	x--;
	break;
/*...e*/
/*...sLST_DIAG_UP:24:*/
case LST_DIAG_UP:
	if ( x < lsect[lsi].len )
		{
		(*yStart) -= x;
		lsect[lsi].len -= x;
		x = 0;
		}
	else
		{
		(*yStart) -= lsect[lsi].len;
		x -= lsect[lsi].len;
		lsi++;
		}
	break;
/*...e*/
/*...sLST_DIAG_DOWN:24:*/
case LST_DIAG_DOWN:
	if ( x < lsect[lsi].len )
		{
		(*yStart) += x;
		lsect[lsi].len -= x;
		x = 0;
		}
	else
		{
		(*yStart) += lsect[lsi].len;
		x -= lsect[lsi].len;
		lsi++;
		}
	break;
/*...e*/
			}
	(*clsect) -= lsi;
	if ( lsi > 0 )
		memcpy(lsect, lsect+lsi, (*clsect)*sizeof(LSECT));
	memcpy(yox, yox+xScroll, (CX_ACTION+CX_EXTRA-xScroll)*sizeof(YOX));
	}
/*...e*/
/*...sGrowRight:0:*/
/*
We must not let the landscape sections, get too long, go offscreen, or get
close to each other. Also, they should not (not must not) be too short.
There may also be constraints as to what can follow what.
Finally, we should randomly change them.
*/

/*...sRandomLSTYPE:0:*/
static LSTYPE RandomLSTYPE(int prob[])
	{
	int i, r = (Random()%100);
	for ( i = 0; i < 5; i++ )
		if ( r < prob[i] )
			return probConvert[i];
		else
			r -= prob[i];
	debug("@@@ Shouldn't get here");
	return LST_FLAT; /* Should never reach here */
	}
/*...e*/
/*...sRandomLen:0:*/
static int RandomLen(int iMinLen, int iMaxLen)
	{
	if ( iMinLen >= iMaxLen )
		return iMaxLen; /* Prevent modulo 0 on next line */
	return iMinLen + ( Random()%(iMaxLen-iMinLen) );
	}
/*...e*/
/*...sIsTooShort:0:*/
static BOOL IsTooShort(LSECT *l)
	{
	switch ( l->lst )
		{
		case LST_FLAT:
			return l->len < level->iMinLenFlat;
		case LST_DIAG_UP:
		case LST_DIAG_DOWN:
			return l->len < level->iMinLenDiag;
		}
	return FALSE;
	}
/*...e*/
/*...sIsTooLong:0:*/
static BOOL IsTooLong(LSECT *l, int y)
	{
	switch ( l->lst )
		{
		case LST_FLAT:
			return l->len >= level->iMaxLenFlat;
		case LST_DIAG_UP:
			return l->len >= level->iMaxLenDiag || y <= 10;
		case LST_DIAG_DOWN:
			return l->len >= level->iMaxLenDiag || y >= CY_ACTION - 10;
		}
	return TRUE;
	}
/*...e*/
#define	abs(x)	(((x)>=0)?(x):-(x))

static void GrowRight(
	int *clsect, LSECT lsect[],
	YOX yox[], int *yEnd,
	int yAvoid,
	int x,
	BOOL bTop
	)
	{
	LSECT *lsectEnd = lsect+(*clsect)-1;
	BOOL bTooLong  = IsTooLong (lsectEnd, (*yEnd));
	BOOL bTooShort = IsTooShort(lsectEnd);
	BOOL bChange = ( (int) (Random()%100) < level->iFreqChange );
	int iApart = abs((*yEnd)-yAvoid);
	BOOL bMinDist = ( iApart <= level->iMinDistApart );
	BOOL bMaxDist = ( iApart >= level->iMaxDistApart );
	LSECT lsectTmp;

	if ( bTooLong || (bChange&&!bTooShort) || bMinDist || bMaxDist )
/*...spick a new one:16:*/
{
int big, iMinLen, iMaxLen;
LSECT *lsectAdd = lsect+(*clsect);
LSTYPE lstAllowed = LST_FLAT;
LSTYPE lstEnd = lsectEnd->lst;
if ( (*yEnd) >=            20 ) lstAllowed |= (LST_VERT_UP  |LST_DIAG_UP  );
if ( (*yEnd) < CY_ACTION - 20 ) lstAllowed |= (LST_VERT_DOWN|LST_DIAG_DOWN);
/*...sif was vertical last time\44\ then can\39\t be vertical this time:16:*/
if ( lstEnd & (LST_VERT_UP|LST_VERT_DOWN) )
	lstAllowed &= ~(LST_VERT_UP|LST_VERT_DOWN);
/*...e*/
/*...savoid vertical and diagonal one after another:16:*/
if ( lstEnd & LST_VERT_UP   ) lstAllowed &= ~LST_DIAG_DOWN;
if ( lstEnd & LST_VERT_DOWN ) lstAllowed &= ~LST_DIAG_UP  ;
if ( lstEnd & LST_DIAG_UP   ) lstAllowed &= ~LST_VERT_DOWN;
if ( lstEnd & LST_DIAG_DOWN ) lstAllowed &= ~LST_VERT_UP  ;
/*...e*/
/*...sif close to yAvoid\44\ then can\39\t approach closer:16:*/
if ( bMinDist )
	{
	if ( (*yEnd) < yAvoid )
		lstAllowed &= ~(LST_VERT_DOWN|LST_DIAG_DOWN);
	else
		lstAllowed &= ~(LST_VERT_UP  |LST_DIAG_UP  );
	}
/*...e*/
/*...sif far from yAvoid\44\ then can\39\t go any further away:16:*/
if ( bMaxDist )
	{
	if ( (*yEnd) < yAvoid )
		lstAllowed &= ~(LST_VERT_UP  |LST_DIAG_UP  );
	else
		lstAllowed &= ~(LST_VERT_DOWN|LST_DIAG_DOWN);
	}
/*...e*/
do
	lsectAdd->lst = RandomLSTYPE(bTop ? level->probTop : level->probBot);
while ( (lsectAdd->lst & lstAllowed) == 0 );
lsectAdd->rgb     = level->rgb;
lsectAdd->rgbEdge = level->rgbEdge;
switch ( lsectAdd->lst )
	{
	case LST_FLAT:
	case LST_DIAG_UP:
	case LST_DIAG_DOWN:
		lsectAdd->len = 1;
		break;
	case LST_VERT_UP:
		big = (*yEnd) - 10;
		if ( big < 0 ) big = 0;
		iMinLen = min(level->iMinLenVert,big);
		iMaxLen = min(level->iMaxLenVert,big);
		if ( !bTop )
			{
			int iGap = iApart - level->iMinDistApart;
			if ( iGap < 0 ) iGap = 0;
			iMinLen = min(iMinLen, iGap);
			iMaxLen = min(iMaxLen, iGap);
			}
		lsectAdd->len = RandomLen(iMinLen, iMaxLen);
		break;
	case LST_VERT_DOWN:
		big = CY_ACTION - 10 - (*yEnd);
		if ( big < 0 ) big = 0;
		iMinLen = min(level->iMinLenVert,big);
		iMaxLen = min(level->iMaxLenVert,big);
		if ( bTop )
			{
			int iGap = iApart - level->iMinDistApart;
			if ( iGap < 0 ) iGap = 0;
			iMinLen = min(iMinLen, iGap);
			iMaxLen = min(iMaxLen, iGap);
			}
		lsectAdd->len = RandomLen(iMinLen, iMaxLen);
		break;
	}
(*clsect)++;
lsectEnd++;
}
/*...e*/
	else
		lsectEnd->len++;
	lsectTmp = *lsectEnd;
	switch ( lsectTmp.lst )
		{
		case LST_FLAT:
		case LST_DIAG_UP:
		case LST_DIAG_DOWN:
			lsectTmp.len = 1;
			break;
		}
	if ( bTop )
		(*yEnd) = ExpandTopYs(1, &lsectTmp, &yox[x], (*yEnd));
	else
		(*yEnd) = ExpandBotYs(1, &lsectTmp, &yox[x], (*yEnd));
	}	
/*...e*/
/*...sMaxInArray:0:*/
static int MaxInArray(YOX array[], int size)
	{
	int i, m = -0x7fff;
	for ( i = 0; i < size; i++ )
		if ( array[i].y > m )
			m = array[i].y;
	return m;
	}
/*...e*/
/*...sMinInArray:0:*/
static int MinInArray(YOX array[], int size)
	{
	int i, m = 0x7fff;
	for ( i = 0; i < size; i++ )
		if ( array[i].y < m )
			m = array[i].y;
	return m;
	}
/*...e*/

#define	CX_LOOKBACK 40

static void ScrollLeft(int xScroll)
	{
	int x;
	int yAvoidTop = MinInArray(yoxBot+CX_ACTION+CX_EXTRA-CX_LOOKBACK, CX_LOOKBACK);
	int yAvoidBot = MaxInArray(yoxTop+CX_ACTION+CX_EXTRA-CX_LOOKBACK, CX_LOOKBACK);
	ConsumeLeft(&clsectTop, lsectTop, &yStartTop, yoxTop, xScroll);
	ConsumeLeft(&clsectBot, lsectBot, &yStartBot, yoxBot, xScroll);
	for ( x = CX_ACTION+CX_EXTRA-xScroll; x < CX_ACTION+CX_EXTRA; x++ )
		{
		GrowRight(&clsectTop, lsectTop, yoxTop, &yEndTop, yAvoidTop, x, TRUE);
		if ( yEndTop > yAvoidBot ) yAvoidBot = yEndTop;
		GrowRight(&clsectBot, lsectBot, yoxBot, &yEndBot, yAvoidBot, x, FALSE);
		if ( yEndBot < yAvoidTop ) yAvoidTop = yEndBot;
		}
	}
/*...e*/
/*...sScrollLeftScreen:0:*/
static void ScrollLeftScreen(int xScroll, HDC hdc)
	{
	LONG rgbBlack      = RGB(0,0,0);
	HPEN hpenBlack     = CreatePen(PS_SOLID, 1, rgbBlack);
	HBRUSH hbrushBlack = CreateSolidBrush(rgbBlack);
	int i, x;

	LONG rgb      = lsectTop[0].rgb;
	HPEN hpen     = CreatePen(PS_SOLID, 1, rgb    );
	LONG rgbEdge  = lsectTop[0].rgbEdge;
	HPEN hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);
	HBRUSH hbrush = CreateSolidBrush(rgb);

	SetWindowOrgEx(hdc, CX_EXTRA/2, 0, NULL);

	for ( i = 0, x = 0; i < clsectTop; i++ )
		{
/*...scolour changes:16:*/
if ( lsectTop[i].rgb != rgb )
	{
	HPEN hpenOld; HBRUSH hbrushOld;
	rgb = lsectTop[i].rgb;
	DeleteObject(hpen);
	hpen = CreatePen(PS_SOLID, 1, rgb);
	DeleteObject(hbrush);
	hbrush = CreateSolidBrush(rgb);
	hpenOld = SelectObject(hdc, hpen);
	hbrushOld = SelectObject(hdc, hbrush);
	Rectangle(hdc, x, 0, x+xScroll, yoxTop[x].y-1);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	}
if ( lsectTop[i].rgbEdge != rgbEdge )
	{
	HPEN hpenOld;
	rgbEdge = lsectTop[i].rgbEdge;
	DeleteObject(hpenEdge);
	hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge); 
	hpenOld = SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x+xScroll+LINEEND, yoxTop[x].y-1);
	SelectObject(hdc, hpenOld);
	}
/*...e*/
		switch ( lsectTop[i].lst )
			{
/*...sLST_FLAT:24:*/
case LST_FLAT:
	x += lsectTop[i].len;
	break;
/*...e*/
/*...sLST_VERT_UP:24:*/
case LST_VERT_UP:
	{
	int len          = lsectTop[i].len;
	HPEN hpenOld     = SelectObject(hdc, hpenBlack);
	HBRUSH hbrushOld = SelectObject(hdc, hbrushBlack);
	Rectangle(hdc, x+1, yoxTop[x].y-len, x+1+xScroll, yoxTop[x].y);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x, yoxTop[x].y-len-1);
	LineTo(hdc, x+xScroll-1+LINEEND, yoxTop[x].y-len-1);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	x++;
	}
	break;
/*...e*/
/*...sLST_VERT_DOWN:24:*/
case LST_VERT_DOWN:
	{
	int len       = lsectTop[i].len;
	HPEN hpenOld  = SelectObject(hdc, hpen);
	HBRUSH hbrushOld = SelectObject(hdc, hbrush);
	Rectangle(hdc, x+1, yoxTop[x].y-len-1, x+1+xScroll, yoxTop[x].y-1);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxTop[x].y-len-1, NULL);
	LineTo(hdc, x, yoxTop[x].y-1);
	LineTo(hdc, x+xScroll+LINEEND, yoxTop[x].y-1);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	x++;
	}
	break;
/*...e*/
/*...sLST_DIAG_UP:24:*/
case LST_DIAG_UP:
	{
	int len      = lsectTop[i].len;
	HPEN hpenOld = SelectObject(hdc, hpenEdge);
	int j;
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x+len, yoxTop[x].y-1-len);
	LineTo(hdc, x+len+(xScroll-1)+LINEEND, yoxTop[x].y-1-len);
	SelectObject(hdc, hpenBlack);
	for ( j = 1; j <= xScroll; j++ )
		{
		MoveToEx(hdc, x+j, yoxTop[x].y-1, NULL);
		LineTo(hdc, x+j+(len-1)+LINEEND, yoxTop[x].y-1-(len-1)-LINEEND);
		}
	SelectObject(hdc, hpenOld);
	x += lsectTop[i].len;
	}
	break;
/*...e*/
/*...sLST_DIAG_DOWN:24:*/
case LST_DIAG_DOWN:
	{
	int len      = lsectTop[i].len;
	HPEN hpenOld = SelectObject(hdc, hpenEdge);
	int j;
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x+(len-1), yoxTop[x].y-1+(len-1));
	LineTo(hdc, x+(len-1)+xScroll-1+LINEEND, yoxTop[x].y-1+(len-1));
	SelectObject(hdc, hpen);
	for ( j = 0; j < xScroll; j++ )
		{
		MoveToEx(hdc, x+j, yoxTop[x].y-2, NULL);
		LineTo(hdc, x+j+(len-1)+LINEEND, yoxTop[x].y-2+(len-1)+LINEEND);
		}
	SelectObject(hdc, hpenOld);
	x += lsectTop[i].len;
	}
	break;
/*...e*/
			}
		}

	for ( i = 0, x = 0; i < clsectBot; i++ )
		{
/*...scolour changes:16:*/
if ( lsectBot[i].rgb != rgb )
	{
	HPEN hpenOld; HBRUSH hbrushOld;
	rgb = lsectBot[i].rgb;
	DeleteObject(hpen);
	hpen = CreatePen(PS_SOLID, 1, rgb);
	DeleteObject(hbrush);
	hbrush = CreateSolidBrush(rgb);
	hpenOld = SelectObject(hdc, hpen);
	hbrushOld = SelectObject(hdc, hbrush);
	Rectangle(hdc, x, yoxBot[x].y+1, x+xScroll, CY_ACTION);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	}
if ( lsectBot[i].rgbEdge != rgbEdge )
	{
	HPEN hpenOld;
	rgbEdge = lsectBot[i].rgbEdge;
	DeleteObject(hpenEdge);
	hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge); 
	hpenOld = SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x+xScroll+LINEEND, yoxBot[x].y);
	SelectObject(hdc, hpenOld);
	}
/*...e*/
		switch ( lsectBot[i].lst )
			{
/*...sLST_FLAT:24:*/
case LST_FLAT:
	x += lsectBot[i].len;
	break;
/*...e*/
/*...sLST_VERT_UP:24:*/
case LST_VERT_UP:
	{
	int len          = lsectBot[i].len;
	HPEN hpenOld     = SelectObject(hdc, hpen);
	HBRUSH hbrushOld = SelectObject(hdc, hbrush);
	Rectangle(hdc, x+1, yoxBot[x].y+1, x+1+xScroll, yoxBot[x].y+len+1);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxBot[x].y+len, NULL);
	LineTo(hdc, x, yoxBot[x].y);
	LineTo(hdc, x+xScroll+LINEEND, yoxBot[x].y);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	x++;
	}
	break;
/*...e*/
/*...sLST_VERT_DOWN:24:*/
case LST_VERT_DOWN:
	{
	int len          = lsectBot[i].len;
	HPEN hpenOld     = SelectObject(hdc, hpenBlack);
	HBRUSH hbrushOld = SelectObject(hdc, hbrushBlack);
	Rectangle(hdc, x+1, yoxBot[x].y, x+1+xScroll, yoxBot[x].y+len);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x, yoxBot[x].y+len);
	LineTo(hdc, x+xScroll-1+LINEEND, yoxBot[x].y+len);
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	x++;
	}
	break;
/*...e*/
/*...sLST_DIAG_UP:24:*/
case LST_DIAG_UP:
	{
	int len      = lsectBot[i].len;
	HPEN hpenOld = SelectObject(hdc, hpenEdge);
	int j;
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x+(len-1), yoxBot[x].y-(len-1));
	LineTo(hdc, x+(len-1)+xScroll-1+LINEEND, yoxBot[x].y-(len-1));
	SelectObject(hdc, hpen);
	for ( j = 0; j < xScroll; j++ )
		{
		MoveToEx(hdc, x+j, yoxBot[x].y+1, NULL);
		LineTo(hdc, x+j+(len-1)+LINEEND, yoxBot[x].y+1-(len-1)-LINEEND);
		}
	SelectObject(hdc, hpenOld);
	x += lsectBot[i].len;
	}
	break;
/*...e*/
/*...sLST_DIAG_DOWN:24:*/
case LST_DIAG_DOWN:
	{
	int len      = lsectBot[i].len;
	HPEN hpenOld = SelectObject(hdc, hpenEdge);
	int j;
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x+len, yoxBot[x].y+len);
	LineTo(hdc, x+len+(xScroll-1)+LINEEND, yoxBot[x].y+len);
	SelectObject(hdc, hpenBlack);
	for ( j = 1; j <= xScroll; j++ )
		{
		MoveToEx(hdc, x+j, yoxBot[x].y, NULL);
		LineTo(hdc, x+j+(len-1)+LINEEND, yoxBot[x].y+(len-1)+LINEEND);
		}
	SelectObject(hdc, hpenOld);
	x += lsectBot[i].len;
	}
	break;
/*...e*/
			}
		}

	DeleteObject(hpen);
	DeleteObject(hbrush);
	DeleteObject(hpenEdge);

	DeleteObject(hpenBlack);
	DeleteObject(hbrushBlack);

	SetWindowOrgEx(hdc, 0, 0, NULL);
	}

/*...e*/
/*...sIncreaseFuel:0:*/
static void IncreaseFuel(int iExtra)
	{
	iFuel += iExtra;
	if ( iFuel > MAX_FUEL )
		iFuel = MAX_FUEL;
	}
/*...e*/
/*...sRectOverlapsRect:0:*/
static BOOL RectOverlapsRect(
	int x1, int y1, int cx1, int cy1,
	int x2, int y2, int cx2, int cy2
	)	
	{
	return x2<x1+cx1 && x1<x2+cx2 && y2<y1+cy1 && y1<y2+cy2;
	}
/*...e*/
/*...sRectOverlapsShip:0:*/
static BOOL RectOverlapsShip(int x, int y, int cx, int cy)
	{
	return RectOverlapsRect(x,y,cx,cy, xShip+X_HIT_SHIP1,yShip+Y_HIT_SHIP1,CX_HIT_SHIP1,CX_HIT_SHIP1) ||
	       RectOverlapsRect(x,y,cx,cy, xShip+X_HIT_SHIP2,yShip+Y_HIT_SHIP2,CX_HIT_SHIP2,CX_HIT_SHIP2);
	}
/*...e*/
/*...sExplodeShip:0:*/
static void ExplodeShip(void)
	{
	if ( bDead )
		return; /* Already exploding! */

	yShip -= ((32-CY_SHIP)/2);
	bDead = TRUE;
	iDeathState = 0;
	}
/*...e*/
/*...sMoveShip:0:*/
static void MoveShip(void)
	{
	AREA *areaShip;
	if ( bDead )
/*...smove and explode as we go:16:*/
{
xShip -= iScrollBy;
areaShip = ConstructArea(xShip-(CX_EXTRA/2), yShip, CX_SHIP, 32,
	CX_SHIP, sprites[SPR_32X32E+iDeathState++].hicon);
}
/*...e*/
	else if ( iMode == MODE_DEMO )
/*...smove by computer\44\ as in demo mode:16:*/
{
int yTop = MaxTopY(xShip-10, xShip+CX_SHIP+20);
int yBot = MinBotY(xShip-10, xShip+CX_SHIP+20);
int yAim = (yTop+yBot)/2;

if ( yShip > yAim+10 )
	yShip -= iScrollBy;
else if ( yShip < yAim-10 )
	yShip += iScrollBy;

if ( HitLand(xShip+X_HIT_SHIP1, yShip+Y_HIT_SHIP1, CX_HIT_SHIP1, CY_HIT_SHIP1) ||
     HitLand(xShip+X_HIT_SHIP2, yShip+Y_HIT_SHIP2, CX_HIT_SHIP2, CY_HIT_SHIP2) )
	ExplodeShip();
areaShip = ConstructArea(xShip-(CX_EXTRA/2), yShip, CX_SHIP, CY_SHIP,
	CX_HIT_SHIP2-2, sprites[SPR_SHIP].hicon);
}
/*...e*/
	else
/*...smove by user \39\cos alive:16:*/
{
if ( bUpPressed )
	yShip -= iScrollBy;
if ( bDownPressed )
	yShip += iScrollBy;
/*...sMax out of sky:16:*/
if ( iFuel == 0 )
	yShip += 3;
if ( iFuel < MAX_FUEL/30 )
	yShip += 2;
if ( iFuel < MAX_FUEL/20 )
	yShip++;
/*...e*/
if ( bLeftPressed )
	{
	if ( xShip > X_SHIP_INIT-40 )
		xShip -= iScrollBy;
	}
else if ( xShip < X_SHIP_INIT && (iCycle&1)!=0 )
	xShip++;
if ( bRightPressed )
	{
	if ( xShip < X_SHIP_INIT+50 )
		xShip += iScrollBy;
	}
else if ( xShip > X_SHIP_INIT && (iCycle&1)!=0 )
	xShip--;

if ( HitLand(xShip+X_HIT_SHIP1, yShip+Y_HIT_SHIP1, CX_HIT_SHIP1, CY_HIT_SHIP1) ||
     HitLand(xShip+X_HIT_SHIP2, yShip+Y_HIT_SHIP2, CX_HIT_SHIP2, CY_HIT_SHIP2) )
	ExplodeShip();
areaShip = ConstructArea(xShip-(CX_EXTRA/2), yShip, CX_SHIP, CY_SHIP,
	CX_HIT_SHIP2-2, sprites[SPR_SHIP].hicon);
}
/*...e*/
	if ( areaShip != NULL )
		{
		areaShip->areaNext = areaDraw;
		areaDraw = areaShip;
		}
	}
/*...e*/
/*...sbaddy data:0:*/
#define	BT_NULL		0
#define	BT_EXPLOSION	1
#define	BT_LANDITEM	2
#define	BT_ROCKET	3
#define	BT_MISSILE	4
#define	BT_UD		5
#define	BT_MINE		6
#define	BT_LSHIP	7
#define	BT_ACIDSHIP	8
#define	BT_ACID		9
#define	BT_BOUNCE	10
#define	BT_SHUT		11
#define	BT_ASTER	12
#define	BT_ROTOR	13
#define	BT_ROTORD	14
#define	BT_CUTE		15
#define	BT_BONUS	16

typedef struct
	{
	int x, y, cx, cy;
	int xHit, yHit, cxHit, cyHit;	/* Rectangle for laser hit testing */
	int xTouch, yTouch, cxTouch, cyTouch; /* Collision testing */
	int type, type2;
	int state, state2;
	} BADDY;

#define	MAX_BADDYS 20
static BADDY baddys[MAX_BADDYS];
/*...e*/
/*...sNoBaddys:0:*/
static void NoBaddys(void)
	{
	int i;
	for ( i = 0; i < MAX_BADDYS; i++ )
		baddys[i].type = BT_NULL;
	}
/*...e*/
/*...slanditem data:0:*/
#define	BT2_FUEL	0
#define	BT2_FUEL2	1
#define	BT2_ARIEL	2
#define	BT2_NUKE	3

typedef struct { int iScore, iSprNum; } LANDITEM;

static LANDITEM landitems[] =
	{
	 { 10, SPR_FUEL  },
	 { 15, SPR_FUEL2 },
	 { 20, SPR_ARIEL },
	 {  0, SPR_NUKE  },
	};
#define	N_LANDITEMS (sizeof(landitems)/sizeof(landitems[0]))

/*...e*/
/*...sFireLaser:0:*/
#define	CX_LASER 420

/*...sHitBaddy:0:*/
static BADDY *HitBaddy(void)
	{
	int i; BADDY *b = baddys;
	for ( i = 0; i < MAX_BADDYS; i++, b++ )
		if ( b->type != BT_NULL      &&
		     b->type != BT_EXPLOSION && 
		     b->type != BT_ACID      &&
		     b->type != BT_BONUS     &&
		     RectOverlapsRect(xLaser, yLaser, cxLaser, 1,
		     b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit) )
			{
			/* Move laser end to go all the way through it */
			cxLaser = b->x+b->xHit+b->cxHit-xLaser;
			return b;
			}
	return NULL;
	}
/*...e*/

static void FireLaser(void)
	{
	AREA *areaLaser;

	bLaserOn = FALSE;
	if ( iLaserHeat > 0 )
		{
		iLaserHeat--;
		return;
		}

	if ( bSpacePressed || iMode == MODE_DEMO )
		{
		xLaser = xShip + CX_SHIP;
		yLaser = yShip + CY_SHIP/2;

		for ( cxLaser = 0; cxLaser < CX_LASER; cxLaser++ )
			if ( yoxTop[xLaser+cxLaser].y > yLaser )
				break;
			else if ( yoxBot[xLaser+cxLaser].y <= yLaser )
				break;
			else if ( (cxLaser&31) == 0 && HitBaddy() != NULL )
				break;

		if ( iMode == MODE_DEMO )
			{
			BADDY *b = HitBaddy();
			if ( b != NULL )
				if (  b->type == BT_NULL ||
				     (b->type == BT_LANDITEM && b->type2 == BT2_NUKE) )
					return;
			}

		bLaserOn = TRUE;
		iLaserHeat = 4;

		areaLaser = ConstructArea(xLaser-(CX_EXTRA/2),yLaser,cxLaser,1,4,NULL);
		if ( areaLaser != NULL )
			{
			areaLaser->areaNext = areaDraw;
			areaDraw = areaLaser;
			}
		}
	}
/*...e*/
/*...sFreeBaddy:0:*/
static BADDY *FreeBaddy(void)
	{
	int i;
	for ( i = 0; i < MAX_BADDYS; i++ )
		if ( baddys[i].type == BT_NULL )
			return &(baddys[i]);
	return NULL;
	}
/*...e*/
/*...sAddToFloor:0:*/
static int iRecentAddFloor = 0;

static BADDY *AddToFloor(int cxClearance)
	{
	BADDY *b;
	LSECT *lsect;
	if ( iRecentAddFloor > 0 )
		{
		iRecentAddFloor--;
		return NULL;
		}
	if ( (b = FreeBaddy()) == NULL )
		return NULL; /* At max number of moving objects */
	lsect = &lsectBot[clsectBot-1];
	if ( lsect->lst != LST_FLAT )
		return NULL; /* Not a flat bit of land */
	if ( lsect->len < cxClearance+4+32+4 )
		return NULL; /* Not enough flatness */
	return b;
	}

static void AddToFloorConfirmed(int cx)
	{
	iRecentAddFloor = cx+4;
	}
/*...e*/
/*...sAddToRoof:0:*/
static int iRecentAddRoof = 0;

static BADDY *AddToRoof(int cxClearance)
	{
	BADDY *b;
	LSECT *lsect;
	if ( iRecentAddRoof > 0 )
		{
		iRecentAddRoof--;
		return NULL;
		}
	if ( (b = FreeBaddy()) == NULL )
		return NULL; /* At max number of moving objects */
	lsect = &lsectTop[clsectBot-1];
	if ( lsect->lst != LST_FLAT )
		return NULL; /* Not a flat bit of land */
	if ( lsect->len < cxClearance+4+32+4 )
		return NULL; /* Not enough flatness */
	return b;
	}

static void AddToRoofConfirmed(int cx)
	{
	iRecentAddRoof = cx+4;
	}
/*...e*/
/*...sAddToAir:0:*/
static int iRecentAddAir = 0;

static BADDY *AddToAir(int cxClearance)
	{
	BADDY *b;
	if ( iRecentAddAir > 0 )
		{
		iRecentAddAir--;
		return NULL;
		}
	if ( (b = FreeBaddy()) == NULL )
		return NULL; /* At max number of moving objects */
	return b;
	}

static void AddToAirConfirmed(int cx)
	{
	iRecentAddAir = cx+4;
	}
/*...e*/
/*...sAddLandItem:0:*/
static void AddLandItem(void)
	{
	BADDY *b;
	LANDITEM *l;

	if ( (b = AddToFloor(0)) == NULL )
		return;
	if ( ( Random()%3 ) != 0 )
		return;

	b->type = BT_LANDITEM;
	b->type2 = ( (Random()>>2) % N_LANDITEMS );
	if ( b->type2 == BT2_NUKE && (Random()&0x80) != 0 )
		b->type2 = BT2_FUEL;
	l = &(landitems[b->type2]);
	b->cx = b->cxHit = sprites[l->iSprNum].cx;
	b->cy = b->cyHit = sprites[l->iSprNum].cy;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->y = yoxBot[b->x].y - b->cy;
	b->xHit = 0;
	b->yHit = 0;
	AddToFloorConfirmed(b->cx);
	}
/*...e*/
/*...sAddBaddy:0:*/
static void AddBaddyForLevel(int iLevel)
	{
	switch ( iLevel )
		{
/*...s0 \45\ add rocket:16:*/
case 0:
	{
	BADDY *b;
	if ( (b = AddToFloor(0)) == NULL )
		return;
	if ( ( Random()%3 ) != 0 )
		return;
	b->type = BT_ROCKET;
	b->state = 0;
	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->y = yoxBot[b->x].y - b->cy;
	b->xHit  = b->xTouch  = 4;
	b->yHit  = b->yTouch  = 2;
	b->cxHit = b->cxTouch = 32-4-4;
	b->cyHit = b->cyTouch = 32-2;
	AddToFloorConfirmed(b->cx);
	}
	break;
/*...e*/
/*...s1 \45\ add missile:16:*/
case 1:
	{
	BADDY *b;
	if ( (b = AddToFloor(20)) == NULL )
		return;
	if ( Random()&0x20 )
		return;
	b->type = BT_MISSILE;
	b->state = 0;
	b->cx = 32;
	b->cy = 18;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->y = yoxBot[b->x].y - b->cy;
	b->xHit  = b->xTouch  = 3;
	b->yHit  = b->yTouch  = 4;
	b->cxHit = b->cxTouch = 32-3-5;
	b->cyHit = b->cyTouch = 7;
	AddToFloorConfirmed(b->cx);
	}
	break;
/*...e*/
/*...s2 \45\ add up and downer\44\ or mine:16:*/
case 2:
	{
	BADDY *b;
	if ( (b = AddToFloor(0)) != NULL && (Random()&0x10) != 0 )
		{
		b->type = BT_UD;
		b->state = 0;
		b->cx = 24;
		b->cy = 32;
		b->x = CX_EXTRA/2 + CX_ACTION;
		b->y = yoxBot[b->x].y - b->cy-10;
		b->xHit  = b-> xTouch  = 0;
		b->yHit  = b-> yTouch  = 0;
		b->cxHit = b-> cxTouch = 24;
		b->cyHit = b-> cyTouch = 32;
		AddToFloorConfirmed(b->cx);
		}
	if ( (Random()%45) == 13 && (b = FreeBaddy()) != NULL )
		{
		int i;
		BADDY *b_ud = NULL;
		for ( i = 0; i < MAX_BADDYS; i++ )
			if ( baddys[i].type == BT_UD )
				if ( b_ud == NULL || (Random()&0x20) == 0 )
					b_ud = &(baddys[i]);
		if ( b_ud != NULL )
			{
			b->type = BT_MINE;
			b->state = 0;
			b->cx = 6;
			b->cy = 6;
			b->x = b_ud->x+12-3;
			b->y = b_ud->y+16-3;
			b->xHit  = b->xTouch  = 0;
			b->yHit  = b->yTouch  = 0;
			b->cxHit = b->cxTouch = 6;
			b->cyHit = b->cyTouch = 6;
			}
		}
	}
	break;
/*...e*/
/*...s3 \45\ add left facing ship:16:*/
case 3:
	{
	BADDY *b;
	if ( (b = AddToFloor(3)) != NULL && (Random()&0x10) != 0 )
		{
		b->type = BT_LSHIP;
		b->state = 0;
		b->cx = 32;
		b->cy = 21;
		b->x = CX_EXTRA/2 + CX_ACTION;
		b->y = yoxBot[b->x].y - b->cy-10;
		b->xHit  = 5;
		b->yHit  = 8;
		b->cxHit = 24;
		b->cyHit = 6;
		b->xTouch  = 5;
		b->yTouch  = 5;
		b->cxTouch = 24;
		b->cyTouch = 10;
		AddToFloorConfirmed(b->cx);
		}
	}
	break;
/*...e*/
/*...s4 \45\ add acid dropper\44\ or acid cloud:16:*/
case 4:
	{
	BADDY *b;
	if ( (b = AddToFloor(3)) != NULL && (Random()&0x10) != 0 )
		{
		b->type = BT_ACIDSHIP;
		b->state = 0;
		b->cx = 32;
		b->cy = 28;
		b->x = CX_EXTRA/2 + CX_ACTION;
		b->y = yoxBot[b->x].y - b->cy-10;
		b->xHit  = 4;
		b->yHit  = 4;
		b->cxHit = 24;
		b->cyHit = 24;
		b->xTouch  = 2;
		b->yTouch  = 2;
		b->cxTouch = 28;
		b->cyTouch = 26;
		AddToFloorConfirmed(b->cx);
		}
	if ( (Random()%30) == 13 && (b = FreeBaddy()) != NULL )
		{
		int i;
		BADDY *b_acid = NULL;
		for ( i = 0; i < MAX_BADDYS; i++ )
			if ( baddys[i].type == BT_ACIDSHIP )
				if ( b_acid == NULL || (Random()&0x20) == 0 )
					b_acid = &(baddys[i]);
		if ( b_acid != NULL )
			{
			b->type = BT_ACID;
			b->state = 0;
			b->cx = 32;
			b->cy = 32;
			b->x = b_acid->x;
			b->y = b_acid->y+24;
			b->xHit  = b->xTouch  = 4;
			b->yHit  = b->yTouch  = 4;
			b->cxHit = b->cxTouch = 32-4-4;
			b->cyHit = b->cyTouch = 32-4-4;
			}
		}
	}
	break;
/*...e*/
/*...s5 \45\ add bouncer:16:*/
case 5:
	{
	BADDY *b;
	if ( (b = AddToFloor(2)) == NULL )
		return;
	if ( ( Random()%3 ) != 0 )
		return;
	b->type = BT_BOUNCE;
	b->state = 0;
	b->state2 = 0;
	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->y = yoxBot[b->x].y - b->cy;
	b->xHit  = 10;
	b->yHit  = 10;
	b->cxHit = 32-10-10;
	b->cyHit = 32-10-10;
	b->xTouch  = 4;
	b->yTouch  = 4;
	b->cxTouch = 32-4-4;
	b->cyTouch = 32-4-4;
	AddToFloorConfirmed(b->cx);
	}
	break;
/*...e*/
/*...s6 \45\ add shuttle or asteroid:16:*/
case 6:
	{
	BADDY *b;
	int y, cy;

	if ( (b = AddToAir(2)) == NULL )
		return;
	if ( ( (Random()>>2)%3 ) != 0 )
		return;

	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	if ( Random()&0x200 )
		/* Add a shuttle */
		{
		b->type = BT_SHUT;
		b->xHit  = 4;
		b->yHit  = 5;
		b->cxHit = 31-4;
		b->cyHit = 30-5;
		b->xTouch  = 4;
		b->yTouch  = 5;
		b->cxTouch = 31-4;
		b->cyTouch = 30-5;
		b->state = 0;
		}
	else
		/* Add an asteroid */
		{
		b->type = BT_ASTER;
		b->xHit  = 5;
		b->yHit  = 5;
		b->cxHit = 32-5-5;
		b->cyHit = 32-5-5;
		b->xTouch  = 5;
		b->yTouch  = 5;
		b->cxTouch = 32-5-5;
		b->cyTouch = 32-5-5;
		b->state = ((Random()>>4)%7);
		}

	y = MaxTopY(b->x, 32);
	cy = MinBotY(b->x, 32) - y - 32;
	if ( cy < 1 ) cy = 1;

	b->y = y + (Random()%cy);

	AddToAirConfirmed(b->cx);
	}
	break;
/*...e*/
/*...s7 \45\ add rotor or rotord:16:*/
case 7:
	{
	BADDY *b;
	int iAdd;

	if ( Random()&0x100 )
		{
		if ( (b = AddToAir(2)) != NULL )
			iAdd = 1;
		else if ( (b = AddToRoof(2)) != NULL )
			iAdd = 0;
		else
			iAdd = 2;
		}
	else
		{
		if ( (b = AddToRoof(2)) != NULL )
			iAdd = 0;
		else if ( (b = AddToAir(2)) != NULL )
			iAdd = 1;
		else
			iAdd = 2;
		}

	switch ( iAdd )
		{
/*...s0 \45\ add rotord:32:*/
case 0:
	{
	if ( ( (Random()>>3)%3 ) == 1 )
		return;

	b->state = 0;
	b->state2 = 0;
	b->xHit  = 5;
	b->yHit  = 5;
	b->cxHit = 32-5-5;
	b->cyHit = 32-5-5;
	b->xTouch  = 5;
	b->yTouch  = 5;
	b->cxTouch = 32-5-5;
	b->cyTouch = 32-5-5;
	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->type = BT_ROTORD;
	b->y = yoxTop[b->x].y;

	AddToRoofConfirmed(b->cx);
	}
	break;
/*...e*/
/*...s1 \45\ add rotor:32:*/
case 1:
	{
	int y, cy;

	if ( ( (Random()>>3)%3 ) != 0 )
		return;

	b->state = 0;
	b->state2 = 0;
	b->xHit  = 5;
	b->yHit  = 5;
	b->cxHit = 32-5-5;
	b->cyHit = 32-5-5;
	b->xTouch  = 5;
	b->yTouch  = 5;
	b->cxTouch = 32-5-5;
	b->cyTouch = 32-5-5;
	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->type = BT_ROTOR;
	y = MaxTopY(b->x, 32);
	cy = MinBotY(b->x, 32) - y - 32;
	if ( cy < 1 ) cy = 1;
	b->y = y + (Random()%cy);

	AddToAirConfirmed(b->cx);
	}
	break;
/*...e*/
		}
	}
	break;
/*...e*/
/*...s8 \45\ add cute guy:16:*/
case 8:
	{
	BADDY *b;
	int y, cy;
	static BOOL bAddLow = FALSE;

	if ( (b = AddToAir(2)) == NULL )
		return;

	b->state = 0;
	b->xHit  = 5;
	b->yHit  = 5;
	b->cxHit = 32-5-5;
	b->cyHit = 32-5-5;
	b->xTouch  = 5;
	b->yTouch  = 5;
	b->cxTouch = 32-5-5;
	b->cyTouch = 32-5-5;
	b->cx = 32;
	b->cy = 32;
	b->x = CX_EXTRA/2 + CX_ACTION;
	b->type = BT_CUTE;
	y = MaxTopY(b->x, 32);
	cy = MinBotY(b->x, 32) - y - 32;

	cy = cy/2;
	if ( bAddLow )
		y += cy;
	bAddLow = !bAddLow;

	if ( cy < 1 ) cy = 1;
	b->y = y + (Random()%cy);

	AddToAirConfirmed(20);
	}
	break;
/*...e*/
		}
	}

static void AddBaddy(void)
	{
	switch ( iLevel )
		{
		case 9:
			{
			static int allowed[] = { 0,1,2,5,7,8 };
			int iLevelRandom = allowed[(Random()>>3)%6];
			AddBaddyForLevel(iLevelRandom);
			}
			break;
		case 10:
			{
			static int allowed[] = { 0,1,2,3,4,6,7 };
			int iLevelRandom = allowed[(Random()>>3)%7];
			AddBaddyForLevel(iLevelRandom);
			}
			break;
		default:
			AddBaddyForLevel(iLevel);
			break;
		}
	}
/*...e*/
/*...sMoveBaddys:0:*/
/*...sDisplayExplosion:0:*/
static void DisplayExplosion(BADDY *b)
	{
	AREA *area;
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[(b->state)++].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMakeExplosion:0:*/
static void MakeExplosion(BADDY *b, int iSprNum, int cSpr)
	{
	b->type = BT_EXPLOSION;
	b->state = iSprNum;
	b->state2 = iSprNum+cSpr;
	DisplayExplosion(b);
	}
/*...e*/
/*...sDisplayBonus:0:*/
static void DisplayBonus(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	switch ( b->state2 )
		{
		case 0: iSprNum = SPR_BON100A;  break;
		case 1: iSprNum = SPR_BON500A;  break;
		case 2: iSprNum = SPR_BON1000A; break;
		}
	iSprNum += ( ( (b->state)++ ) >> 5 );
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/

/*...sMoveExplosion:0:*/
static void MoveExplosion(BADDY *b)
	{
	b->x -= iScrollBy;
	if ( b->state != b->state2 && b->x >= 0 )
		{
		DisplayExplosion(b);
		return;
		}
	if ( (Random()&0x1f) != 12 || b->x < CX_EXTRA || b->cx != 32 || b->cy != 32 )
		{
		b->type = BT_NULL;
		return;
		}

	b->type = BT_BONUS;
	b->state = 0;
	b->state2 = (Random()%3);
	}
/*...e*/
/*...sMoveLandItem:0:*/
static void MoveLandItem(BADDY *b)
	{
	LANDITEM *l = &(landitems[b->type2]);
	AREA *area;
	b->x -= iScrollBy;
	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}
	if ( bLaserOn && RectOverlapsRect(b->x,b->y,b->cx,b->cy,xLaser,yLaser,cxLaser,1) )
		{
		iScore += l->iScore;
		switch ( b->type2 )
			{
			case BT2_FUEL:
				IncreaseFuel(MAX_FUEL/5);
				break;
			case BT2_FUEL2:
				IncreaseFuel(MAX_FUEL/7);
				break;
			case BT2_NUKE:
				ExplodeShip();
				break;
			}
		b->y += b->cy;
		b->y -= 32;
		b->cx = 32;
		b->cy = 32;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[l->iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveRocket:0:*/
static void MoveRocket(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	b->x -= iScrollBy;
	switch ( b->state )
		{
/*...s0 \45\ waiting to launch:16:*/
case 0:
	if ( (Random() % 100) == 27 )
		b->state = 1;
	iSprNum = SPR_ROCKET1;
	break;
/*...e*/
/*...s1 \45\ flying upwards \40\slowly\41\:16:*/
case 1:
	if ( b->y-1 > MaxTopY(b->x-2, 2+32+2)+5 )
		(b->y)--;
	else
		b->state = 2;
	iSprNum = SPR_ROCKET2;
	break;
/*...e*/
/*...s2 \45\ stopped:16:*/
case 2:
	iSprNum = ((Random()&0x40)!=0) ? SPR_ROCKET1: SPR_ROCKET2;
	break;
/*...e*/
		}

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 10;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveMissile:0:*/
static void MoveMissile(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	b->x -= iScrollBy;
	switch ( b->state )
		{
/*...s0 \45\ waiting to launch:16:*/
case 0:
	if ( (Random() % 100) == 27 && yShip <= b->y )
		{
		b->state = 1;
		b->cy = 18;
		b->type2 = (Random()&3);
		}
	iSprNum = SPR_MISSILEG;
	break;
/*...e*/
/*...s1 \45\ flying towards ship:16:*/
case 1:
	{
	int yAim = yShip + b->type2;
	if ( yAim < b->y && b->y-1 > MaxTopY(b->x,32) )
		{
		(b->y)--;
		iSprNum = SPR_MISSILEU;
		b->cy = 16;
		}
	else if ( yAim > b->y && b->y+b->cy+1 < MinBotY(b->x,32) )
		{
		(b->y)++;
		iSprNum = SPR_MISSILED;
		b->cy = 16;
		}
	else
		{
		iSprNum = SPR_MISSILEF;
		b->cy = 15;
		}
	if ( HitLand(b->x-3,b->y,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X15E, 5);
		return;
		}
	(b->x) -= 3;
	}
	break;
/*...e*/
		}

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 11;
		MakeExplosion(b, SPR_32X15E, 5);
		return;
		}

	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveUd:0:*/
static void MoveUd(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	b->x -= iScrollBy;
	switch ( b->state )
		{
/*...s0 \45\ going up:16:*/
case 0:
	if ( b->y-3 > MaxTopY(b->x-2, 2+32+2)+5 )
		(b->y) -= 3;
	else
		b->state = 1;
	break;
/*...e*/
/*...s1 \45\ down:16:*/
case 1:
	if ( b->y+b->cy+3 < MinBotY(b->x-2, 2+32+2)-5 )
		(b->y) += 3;
	else
		b->state = 0;
	break;
/*...e*/
		}

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 12;
		b->cx = 32;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	iSprNum = SPR_UD1 + ((iCycle >> 4) & 3);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveMine:0:*/
static void MoveMine(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	b->x -= iScrollBy;
	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	iSprNum = SPR_MINE1 + ((iCycle >> 3) & 1);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveLship:0:*/
static void MoveLship(BADDY *b)
	{
	AREA *area;
	int iSprNum, yBotShip, yFloor, xNew, yNew;
	b->x -= iScrollBy;

	yBotShip = b->y + b->cy;
	yFloor = MinBotY(b->x-5,5+b->cx+3) - 10;

	if ( yBotShip > yFloor )
		{
		xNew = b->x;
		yNew = b->y - 2;
		iSprNum = SPR_LSHIPFB;
		}
	else if ( yBotShip+2 < yFloor )
		{
		xNew = b->x;
		yNew = b->y + 2;
		iSprNum = SPR_LSHIP;
		}
	else
		{
		xNew = b->x - 3;
		yNew = b->y;
		iSprNum = SPR_LSHIPFR;
		}

	if ( HitLand(xNew, yNew, b->cx, b->cy) )
		{
		b->y += 3;
		b->cy = 15;
		MakeExplosion(b, SPR_32X15E, 5);
		return;
		}

	b->x = xNew;
	b->y = yNew;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 15;
		b->y += 3;
		b->cy = 15;
		MakeExplosion(b, SPR_32X15E, 5);
		return;
		}

	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveAcidShip:0:*/
static void MoveAcidShip(BADDY *b)
	{
	int xNew, yNew;
	AREA *area;
	b->x -= iScrollBy;
	xNew = b->x;
	yNew = b->y;
	switch ( b->state )
		{
/*...s0 \45\ flying upwards \40\quickly\41\:16:*/
case 0:
	if ( b->y-4 > MaxTopY(b->x-4, 4+32+4+iScrollBy)+5 )
		yNew = b->y - 4;
	else
		b->state = (Random()&0x400) ? 1 : 2;
	break;
/*...e*/
/*...s1 \45\ flying left:16:*/
case 1:
	if ( b->y >= MaxTopY(b->x-4, 4+32)+5 )
		xNew = b->x - 3;
	else
		b->state = 2;
	break;
/*...e*/
/*...s2 \45\ flying right:16:*/
case 2:
	if ( b->x >= CX_ACTION+CX_EXTRA/2 )
		b->state = 1;
	else if ( b->y >= MaxTopY(b->x, 32+3+iScrollBy)+5 )
		xNew = b->x + 3;
	else
		b->state = 1;
	break;
/*...e*/
		}

	if ( HitLand(xNew,yNew-2,b->cx,32) )
		{
		(b->y) -= 2;
		b->cy = 32;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	b->x = xNew;
	b->y = yNew;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 16;
		(b->y) -= 2;
		b->cy = 32;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[SPR_ACIDSHIP].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveAcid:0:*/
static void MoveAcid(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	int yFloor;
	b->x -= iScrollBy;

	yFloor = MinBotY(b->x-iScrollBy-2,b->cx+2*iScrollBy+4);
	(b->y) += 4;
	if ( b->y+b->cy > yFloor )
		{
		b->cy = yFloor - b->y;
		if ( b->cy < 2 )
			{
			b->type = BT_NULL;
			return;
			}
		}

	if ( Random()&5 )
		b->state ^= 1;
	iSprNum = SPR_ACID0 + b->state;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveBounce:0:*/
static void MoveBounce(BADDY *b)
	{
	AREA *area;
	int iSprNum;
	b->x -= iScrollBy;
	switch ( b->state )
		{
/*...s0 \45\ going up:16:*/
case 0:
	if ( b->y-4 > MaxTopY(b->x-3, 3+32+3)+3 )
		(b->y) -= 4;
	else
		b->state = 1;
	break;
/*...e*/
/*...s1 \45\ down:16:*/
case 1:
	if ( b->y+b->cy+4 < MinBotY(b->x-3, 3+32+3)-3 )
		(b->y) += 4;
	else
		b->state = 0;
	break;
/*...e*/
		}

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 5;
		if ( ++(b->state2) == 3 )
			{
			IncreaseFuel(MAX_FUEL/3);
			MakeExplosion(b, SPR_32X32E, 5);
			return;
			}
		}

	iSprNum = SPR_BOUNCE1 + (b->state2);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveShut:0:*/
static void MoveShut(BADDY *b)
	{
	AREA *area;
	int iSprNum, dx;
	b->x -= iScrollBy;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	dx = ( b->x < CX_WINDOW/2 ) ? -5 : -3;

	if ( HitLand(b->x+dx,b->y,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	(b->x) += dx;

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 5;
		if ( ++(b->state) == 4 )
			{
			IncreaseFuel(MAX_FUEL/4);
			MakeExplosion(b, SPR_32X32E, 5);
			return;
			}
		}

	iSprNum = SPR_SHUT0 + (b->state);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveAster:0:*/
static void MoveAster(BADDY *b)
	{
	AREA *area;
	int iSprNum, dx, dy;
	b->x -= iScrollBy;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	dx = (Random()%9) - 5;
	dy = (Random()%7) - 3;

	if ( HitLand(b->x+dx,b->y+dy,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	(b->x) += dx;
	(b->y) += dy;

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 5;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	iSprNum = SPR_ASTER0 + (b->state);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveRotor:0:*/
static void MoveRotor(BADDY *b)
	{
	AREA *area;
	int iSprNum, dx, dy, yBot, yTop, yAim;
	b->x -= iScrollBy;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		if ( iScore >= 20 )
			iScore -= 20; /* Ouch! */
		return;
		}

	dx = ( b->x < CX_WINDOW*3/2 ) ? -6 : -2;

	yTop = MaxTopY(b->x+dx, b->cx);
	yBot = MinBotY(b->x+dx, b->cx);
	yAim = (yTop+yBot) / 2;

	if ( b->x < CX_WINDOW/3 )
		yAim = yShip;

	if ( b->y > yAim+10 )
		dy = -2;
	else if ( b->y < yAim-10 )
		dy =  2;
	else
		dy =  0;

	if ( HitLand(b->x+dx,b->y+dy,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	(b->x) += dx;
	(b->y) += dy;

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 15;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	(b->state2)++;

	iSprNum = SPR_ROTOR0 + (((b->state2)>>2)&1);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveRotorD:0:*/
static void MoveRotorD(BADDY *b)
	{
	AREA *area;
	int iSprNum, dx, dy;
	b->x -= iScrollBy;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		return;
		}

	switch ( b->state )
		{
		case 0:
			dx = dy = 0;
			if ( b->x+b->y < xShip+yShip - 20 ||
			     ( ((Random()>>1)%503) < 2 && b->x < CX_WINDOW/2 ) )
				b->state = 1;
			break;
		case 1:
			dx = -1;
			dy = 1 + iScrollBy;
			break;
		}

	if ( HitLand(b->x+dx,b->y+dy,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	(b->x) += dx;
	(b->y) += dy;

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 16;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	(b->state2)++;

	iSprNum = SPR_ROTORD0 + (((b->state2)>>2)&1);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveCute:0:*/
static void MoveCute(BADDY *b)
	{
	AREA *area;
	int iSprNum, dx, dy, yBot, yTop, yAim;
	b->x -= iScrollBy;

	if ( b->x < 0 )
		{
		b->type = BT_NULL;
		if ( iScore >= 30 )
			iScore -= 30; /* Ouch! */
		return;
		}

	dx = ( b->x < CX_WINDOW*3/2 ) ? -8 : -3;

	yTop = MaxTopY(b->x+dx, b->cx);
	yBot = MinBotY(b->x+dx, b->cx);
	yAim = (yTop+yBot) / 2;

	if ( b->x < CX_WINDOW/3 )
		yAim = yShip;

	if ( b->y > yAim+10 )
		dy = -1;
	else if ( b->y < yAim-10 )
		dy =  1;
	else
		dy =  0;

	if ( b->x < CX_WINDOW/2 )
		dy *= 3;
	else
		dy *= 2;

	if ( HitLand(b->x+dx,b->y+dy,b->cx,b->cy) )
		{
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}
	(b->x) += dx;
	(b->y) += dy;

	/* Consider teleporting closer */

	if ( (Random() & 0x3f) == 34 )
		{
		int nx = ( xShip + b->x ) / 2;
		int ny = ( yShip + b->y ) / 2;
		if ( !HitLand(nx-4,ny-4,b->cx+8,b->cy+8) )
			{
			b->x = nx;
			b->y = ny;
			}
		}

	if ( bLaserOn && RectOverlapsRect(b->x+b->xHit,b->y+b->yHit,b->cxHit,b->cyHit,xLaser,yLaser,cxLaser,1) )
		{
		iScore += 20;
		MakeExplosion(b, SPR_32X32E, 5);
		return;
		}

	(b->state2)++;

	iSprNum = SPR_CUTE0 + (((b->state2)>>1)%3);
	if ( (area = ConstructArea(b->x-(CX_EXTRA/2),b->y,b->cx,b->cy,0,sprites[iSprNum].hicon)) == NULL )
		return;
	area->areaNext = areaDraw; areaDraw = area;
	}
/*...e*/
/*...sMoveBonus:0:*/
static void MoveBonus(BADDY *b)
	{
	BOOL bVanish;
	b->x -= iScrollBy;

	if ( RectOverlapsShip(b->x+b->xTouch,b->y+b->yTouch,b->cxTouch,b->cyTouch) )
	     	{
		switch ( b->state2 )
			{
			case 0: iScore +=  10; break;
			case 1: iScore +=  50; break;
			case 2: iScore += 100; break;
			}
		b->type = BT_NULL;
		return;
		}

	switch ( b->state2 )
		{
		case 0: bVanish = ( (b->state>>5) == 2 ); break;
		case 1: bVanish = ( (b->state>>5) == 2 ); break;
		case 2: bVanish = ( (b->state>>5) == 3 ); break;
		}
	if ( !bVanish && b->x >= 0 )
		{
		DisplayBonus(b);
		return;
		}
	b->type = BT_NULL;
	}
/*...e*/

static void MoveBaddys(void)
	{
	BADDY *b = baddys;
	int i;
	for ( i = 0; i < MAX_BADDYS; i++, b++ )
		if ( b->type != BT_NULL )
			{
			if ( b->type != BT_EXPLOSION &&
			     b->type != BT_BONUS     )
				if ( RectOverlapsShip(b->x+b->xTouch,b->y+b->yTouch,b->cxTouch,b->cyTouch) )
					ExplodeShip();
/*...sdirect to MoveXxx function:24:*/
switch ( b->type )
	{
	case BT_EXPLOSION:	MoveExplosion(b);	break;
	case BT_LANDITEM:	MoveLandItem(b);	break;
	case BT_ROCKET:		MoveRocket(b);		break;
	case BT_MISSILE:	MoveMissile(b);		break;
	case BT_UD:		MoveUd(b);		break;
	case BT_MINE:		MoveMine(b);		break;
	case BT_LSHIP:		MoveLship(b);		break;
	case BT_ACIDSHIP:	MoveAcidShip(b);	break;
	case BT_ACID:		MoveAcid(b);		break;
	case BT_BOUNCE:		MoveBounce(b);		break;
	case BT_SHUT:		MoveShut(b);		break;
	case BT_ASTER:		MoveAster(b);		break;
	case BT_ROTOR:		MoveRotor(b);		break;
	case BT_ROTORD:		MoveRotorD(b);		break;
	case BT_CUTE:		MoveCute(b);		break;
	case BT_BONUS:		MoveBonus(b);		break;
	}
/*...e*/
			}
	}
/*...e*/
/*...sOneCycle:0:*/
static void OneCycle(HWND hwnd)
	{
	HDC hdc;
	int iScoreOld = iScore;
	iCycle++;
	areaErase = areaDraw;	/* Area to erase this time is area we drew last time */
	areaDraw = NULL;	/* As yet, have not drawn anything this time */
	ScrollLeft(iScrollBy);
	MoveShip();
	FireLaser();
	AddBaddy();
	AddLandItem();
	MoveBaddys();
	if ( iCycle%3 < 1 && iFuel > 0 )
		iFuel--;
	if ( (iCycle%((CX_ACTION*3)/2)) == CX_ACTION )
		{
		NextLevel();
		IncreaseFuel(MAX_FUEL/2);
		iScore += 50;
		}

	hdc = GetDC(hwnd);
	EraseAreas(areaErase, hdcBacking);
	LandscapeAreas(areaErase, hdcBacking);
	EraseAreas(areaDraw, hdcBacking);
	LandscapeAreas(areaDraw, hdcBacking);
	PlotAreas(areaDraw, hdcBacking);
	ScrollLeftScreen(iScrollBy, hdc);
	RefreshAreas(areaErase, hdcBacking, hdc);
	RefreshAreas(areaDraw, hdcBacking, hdc);
	FreeAreas(areaErase); areaErase = NULL;
	PaintScore(hdc, CX_WINDOW-2-2-6*12, CY_WINDOW-CY_STATUS+2+2, iScore);
	PaintFuelBar(hdc, 2+2+5*12+2, CY_WINDOW-CY_STATUS, iFuel);

	if ( iScore/1000 != iScoreOld/1000 && iLives < 5 )
/*...saward new life:16:*/
{
iLives++;
PaintLives(hdc,
	2+2+5*12+2+MAX_FUEL+2+2+2,
	CY_WINDOW-CY_STATUS+2+2, iLives);		
}
/*...e*/

	ReleaseDC(hwnd, hdc);

	if ( bDead && iDeathState >= 5 )
/*...swhat to do when finish exploding\33\:16:*/
{
if ( --iLives == 0 )
	/* End of game */
	{
	if ( iMode == MODE_GAME )
		{
		if ( iScore > iHiScore )
			iHiScore = iScore;
		}
	iMode = MODE_TITLE;
	lCounter = 0;
	}
else
	/* Another life */
	{
	bDead  = FALSE;
	iCycle = 0;
	iFuel = iFuelLast = MAX_FUEL;
	xShip = X_SHIP_INIT;
	yShip = Y_SHIP_INIT;
	iLaserHeat = 0;
	bLaserOn = FALSE;
	NoBaddys();
	ResetAreas();
	BuildInitLand();
	}
InvalidateRect(hwnd, NULL, TRUE);
UpdateWindow(hwnd);
}
/*...e*/
	}
/*...e*/
/*...sAnimateTitleScreen:0:*/
/*...sCalcAniFrame:0:*/
static int CalcAniFrame(void)
	{
	if ( lCounter < T_IDLE )
		return 0;
	else if ( lCounter < T_IDLE+T_ANI4 )
		return (int) (lCounter-T_IDLE)/4;
	else if ( lCounter < T_IDLE+T_ANI4+T_ANI3 )
		return (int) (lCounter-T_IDLE-T_ANI4)/3;
	else if ( lCounter < T_IDLE+T_ANI4+T_ANI3+T_ANI2 )
		return (int) (lCounter-T_IDLE-T_ANI4-T_ANI3)/2;
	else if ( lCounter < T_IDLE+T_ANI4+T_ANI3+T_ANI2+T_ANI )
		return (int) (lCounter-T_IDLE-T_ANI4-T_ANI3-T_ANI2);
	else
		return 29;
	}
/*...e*/

static void AnimateTitleScreen(HWND hwnd)
	{
	++lCounter;
	if ( lCounter < T_IDLE )
		;
	else if ( lCounter < T_IDLE+T_ANI4+T_ANI3+T_ANI2+T_ANI )
		{
		HDC hdc = GetDC(hwnd);
		int iFrame = CalcAniFrame();
		PaintAniFrame(hdc, iFrame);
		ReleaseDC(hwnd, hdc);
		}
	else
		{
		iMode = MODE_DEMO;
		bDead  = FALSE;
		iLives = 1;
		iCycle = 0;
		iScore = 0;
		SetLevel((Random()>>2)%(N_LEVELS-4));
		BuildInitLand();
		iFuel = iFuelLast = MAX_FUEL;
		xShip = X_SHIP_INIT;
		yShip = Y_SHIP_INIT;
		iLaserHeat = 0;
		bLaserOn = FALSE;
		NoBaddys();
		ResetAreas();
		InvalidateRect(hwnd, NULL, TRUE);
		UpdateWindow(hwnd);
#ifdef USE_MMTIMER
		dwTimeLast = timeGetTime();
		dwTimeCycle = (DWORD) T_CYCLE_INIT;
#endif
		}
	}
/*...e*/
#ifdef USE_MMTIMER
/*...sTimeCallback:0:*/
static void CALLBACK TimeCallback(
	UINT uID,
	UINT uMsg,
	DWORD dwUser,
	DWORD dw1,
	DWORD dw2
	)
	{
	HWND hwnd = (HWND) dwUser;
	PostMessage(hwnd, WM_USER, 0, 0);
	}
/*...e*/
#endif
/*...sAlphaWindowProc:0:*/
/*...sPaintTitle:0:*/
static void PaintTitle(HWND hwnd)
	{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hwnd, &ps);

/*...spaint the whole window:8:*/
ThreeDeeRect(hdc,
	0,0,
	CX_WINDOW,CY_WINDOW,
	RGB(255,255,255), RGB(192,192,192), RGB(128,128,128));
/*...e*/
/*...sadd title bitmap:8:*/
PaintBitmap(hdc, "BitmapTitle", X_TITLE, Y_TITLE, CX_TITLE, CY_TITLE);
/*...e*/
/*...sadd animation window:8:*/
ThreeDeeRect(hdc,
	X_ANI-4, Y_ANI-4,
	X_ANI+CX_ANI+4,Y_ANI+CY_ANI+4,
	RGB(255,255,255), -1L, RGB(128,128,128));
ThreeDeeRect(hdc,
	X_ANI-2, Y_ANI-2,
	X_ANI+CX_ANI+2,Y_ANI+CY_ANI+2,
	RGB(128,128,128), -1L, RGB(255,255,255));

PaintAniFrame(hdc, CalcAniFrame());
/*...e*/
/*...sadd score and hiscore:8:*/
#define	CX_SCORE (7*12+4+6*12)
#define	CY_SCORE 12
#define	X_SCORE (CX_WINDOW-20-CX_SCORE)
#define	Y_SCORE 140
ThreeDeeRect(hdc,
	X_SCORE-4, Y_SCORE-4,
	X_SCORE+CX_SCORE+4,Y_SCORE+CY_SCORE+4,
	RGB(255,255,255), -1L, RGB(128,128,128));
ThreeDeeRect(hdc,
	X_SCORE+7*12+4-2, Y_SCORE-2,
	X_SCORE+CX_SCORE+2,Y_SCORE+CY_SCORE+2,
	RGB(128,128,128), -1L, RGB(255,255,255));

PaintLoadedBitmap(hdc, hbmScore, X_SCORE, Y_SCORE, 5*12, 12);
PaintScore(hdc, X_SCORE+7*12+4, Y_SCORE, iScore);

#define	CX_HISCORE (7*12+4+6*12)
#define	CY_HISCORE 12
#define	X_HISCORE (CX_WINDOW-20-CX_HISCORE)
#define	Y_HISCORE 165
ThreeDeeRect(hdc,
	X_HISCORE-4, Y_HISCORE-4,
	X_HISCORE+CX_HISCORE+4,Y_HISCORE+CY_HISCORE+4,
	RGB(255,255,255), -1L, RGB(128,128,128));
ThreeDeeRect(hdc,
	X_HISCORE+7*12+4-2, Y_HISCORE-2,
	X_HISCORE+CX_HISCORE+2,Y_HISCORE+CY_HISCORE+2,
	RGB(128,128,128), -1L, RGB(255,255,255));

PaintLoadedBitmap(hdc, hbmHiScore, X_HISCORE, Y_HISCORE, 7*12, 12);
PaintScore(hdc, X_HISCORE+7*12+4, Y_HISCORE, iHiScore);
/*...e*/
/*...ssome text at the bottom:8:*/
#define	X_TEXT 20
#define	Y_TEXT 220

{
static char *text[] =
	{
	"Space to start game, arrows to move, space to fire"
						"\t\xa9 Andy Key",
	"Shoot everything except power stations"
						"\t   andy.z.key@googlemail.com",
	"",
	"This game is a little piece of history - retro gamers, enjoy!",
	"The first version was written for the 8-bit Z80 based Memotech MTX computer.",
	"Then it was ported to 16-bit Windows 3.1, back in the days before WinG or DirectX,",
	"using raw GDI, so the amazing thing is that it was fast enough to be playable.",
#ifdef USE_PEEKMSG
	"This is the 32-bit Windows port (using a PeekMessage loop).",
#endif
#ifdef USE_WM_TIMER
	"This is the 32-bit Windows port (using 10ms WM_TIMER messages).",
#endif
#ifdef USE_MMTIMER
	"This is the 32-bit Windows port (using Multimedia Timer).",
#endif
	};
int i;
static int iTabPos = 360;

SetTextColor(hdc, RGB(0,0,0));
SetBkColor(hdc, RGB(192,192,192));
for ( i = 0; i < sizeof(text)/sizeof(text[0]); i++ )
	TabbedTextOut(
		hdc, X_TEXT, Y_TEXT+i*20, text[i], strlen(text[i]),
		1, &iTabPos, X_TEXT);
}
/*...e*/

	EndPaint(hwnd, &ps);
	}
/*...e*/
/*...sPaintGame:0:*/
/*...sPaintStatusBar:0:*/
static void PaintStatusBar(HDC hdc)
	{
	HDC hdcMem = CreateCompatibleDC(hdc);
	HBITMAP hbmOld;

	/* Raised status bar */
	ThreeDeeRect(hdc, 0, 0, CX_WINDOW, CY_STATUS,
		     RGB(255,255,255), RGB(192,192,192), RGB(128,128,128));

	/* Make recess for fuel area */
	ThreeDeeRect(hdc, 2, 2, 2+2+5*12+2+MAX_FUEL+2, CY_STATUS-2,
		     RGB(128,128,128), -1L, RGB(255,255,255));

	/* Write "FUEL" on the left hand edge */
	hbmOld = SelectObject(hdcMem, hbmFuel);
	BitBlt(hdc, 2+2, 2+2, 4*12, 12, hdcMem, 0, 0, SRCCOPY);

	/* Draw fuel bar raised in red */
	ThreeDeeRect(hdc,
		     2+2+5*12+2, 4,
		     2+2+5*12+2+iFuel, CY_STATUS-4,
		     RGB(255,255,255), RGB_FUELBAR, RGB(128,128,128));

	/* Make recess for lives area */
	ThreeDeeRect(hdc,
		     2+2+5*12+2+MAX_FUEL+2+2, 2,
		     CX_WINDOW-2-2-12*12-2-2, CY_STATUS-2,
		     RGB(128,128,128), -1L, RGB(255,255,255));

	PaintLives(hdc, 2+2+5*12+2+MAX_FUEL+2+2+2, 2+2, iLives);

	/* Make recess for score area */
	ThreeDeeRect(hdc,
		     CX_WINDOW-2-2-12*12-2, 2,
		     CX_WINDOW-2, CY_STATUS-2,
		     RGB(128,128,128), -1L, RGB(255,255,255));

	/* Write "SCORE" on the left hand edge */
	SelectObject(hdcMem, hbmScore);
	BitBlt(hdc, CX_WINDOW-2-2-12*12, 2+2, 5*12, 12, hdcMem, 0, 0, SRCCOPY);

	SelectObject(hdcMem, hbmOld);
	DeleteDC(hdcMem);

	PaintScore(hdc, CX_WINDOW-2-2-6*12, 2+2, iScore);
	}
/*...e*/

static void PaintGame(HWND hwnd)
	{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hwnd, &ps);
	int left   = ps.rcPaint.left;
	int right  = ps.rcPaint.right;
	int top    = ps.rcPaint.top;
	int bottom = ps.rcPaint.bottom;

	LONG rgb      = lsectTop[0].rgb;
	HPEN hpen     = CreatePen(PS_SOLID, 1, rgb);
	HBRUSH hbrush = CreateSolidBrush(rgb);
	LONG rgbEdge  = lsectTop[0].rgbEdge;
	HPEN hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);

/*...swipe background to black:8:*/
{
HPEN hpen = SelectObject(hdc, CreatePen(PS_SOLID,1, RGB(0,0,0)));
HBRUSH hbrush = SelectObject(hdc, CreateSolidBrush(RGB(0,0,0)));
Rectangle(hdc, left,top,right,bottom);
DeleteObject(SelectObject(hdc, hbrush));
DeleteObject(SelectObject(hdc, hpen));
}
/*...e*/

	SetWindowOrgEx(hdc, CX_EXTRA/2, 0, NULL);

/*...sdo top half:8:*/
{
int i, j, x, y1, y2, len;
for ( i = 0, x = 0; i < clsectTop; i++ )
	{
	HPEN hpenOld; HBRUSH hbrushOld;
	if ( lsectTop[i].rgb != rgb )
/*...schange hpen and hbrush:24:*/
{
rgb = lsectTop[i].rgb;
DeleteObject(hpen);   hpen   = CreatePen(PS_SOLID, 1, rgb);
DeleteObject(hbrush); hbrush = CreateSolidBrush(rgb);
}
/*...e*/
	if ( lsectTop[i].rgbEdge != rgbEdge )
/*...schange hpenEdge:24:*/
{
rgbEdge = lsectTop[i].rgbEdge;
DeleteObject(hpenEdge); hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);
}
/*...e*/
	hpenOld   = SelectObject(hdc, hpen);
	hbrushOld = SelectObject(hdc, hbrush);
	len = lsectTop[i].len;
	switch ( lsectTop[i].lst )
		{
/*...sLST_FLAT:24:*/
case LST_FLAT:
	y1 = yoxTop[x].y;
	Rectangle(hdc, x, 0, x+len, y1-1);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y1-1, NULL);
	LineTo(hdc, x+len-1+LINEEND, y1-1);
	x += len;
	break;
/*...e*/
/*...sLST_VERT_UP:24:*/
case LST_VERT_UP:
	y1 = yoxTop[x].y; y2 = y1 - len;
	MoveToEx(hdc, x, 0, NULL);
	LineTo(hdc, x, y2-2+LINEEND);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y2-1, NULL);
	LineTo(hdc, x, y1-1+LINEEND);
	x++;
	break;
/*...e*/
/*...sLST_VERT_DOWN:24:*/
case LST_VERT_DOWN:
	y1 = yoxTop[x].y; y2 = y1 - len;
	MoveToEx(hdc, x, 0, NULL);
	LineTo(hdc, x, y2-2+LINEEND);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y2-1, NULL);
	LineTo(hdc, x, y1-1+LINEEND);
	x++;
	break;
/*...e*/
/*...sLST_DIAG_UP:24:*/
case LST_DIAG_UP:
	for ( j = 0; j < len; j++ )
		{
		MoveToEx(hdc, x+j, 0, NULL);
		LineTo(hdc, x+j, yoxTop[x+j].y-1+LINEEND);
		}
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x+len-1+LINEEND, yoxTop[x+len-1].y-1-LINEEND);
	x += len;
	break;
/*...e*/
/*...sLST_DIAG_DOWN:24:*/
case LST_DIAG_DOWN:
	for ( j = 0; j < len; j++ )
		{
		MoveToEx(hdc, x+j, 0, NULL);
		LineTo(hdc, x+j, yoxTop[x+j].y-1+LINEEND);
		}
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxTop[x].y-1, NULL);
	LineTo(hdc, x+len-1+LINEEND, yoxTop[x+len-1].y-1+LINEEND);
	x += len;
	break;
/*...e*/
		}
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	}
}
/*...e*/
/*...sdo bot half:8:*/
{
int i, j, x, y1, y2, len;
for ( i = 0, x = 0; i < clsectBot; i++ )
	{
	HPEN hpenOld; HBRUSH hbrushOld;
	if ( lsectBot[i].rgb != rgb )
/*...schange hpen and hbrush:24:*/
{
rgb = lsectBot[i].rgb;
DeleteObject(hpen);   hpen   = CreatePen(PS_SOLID, 1, rgb);
DeleteObject(hbrush); hbrush = CreateSolidBrush(rgb);
}
/*...e*/
	if ( lsectBot[i].rgbEdge != rgbEdge )
/*...schange hpenEdge:24:*/
{
rgbEdge = lsectBot[i].rgbEdge;
DeleteObject(hpenEdge); hpenEdge = CreatePen(PS_SOLID, 1, rgbEdge);
}
/*...e*/
	hpenOld   = SelectObject(hdc, hpen);
	hbrushOld = SelectObject(hdc, hbrush);
	len = lsectBot[i].len;
	switch ( lsectBot[i].lst )
		{
/*...sLST_FLAT:24:*/
case LST_FLAT:
	y1 = yoxBot[x].y;
	Rectangle(hdc, x, y1+1, x+len, CY_ACTION);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y1, NULL);
	LineTo(hdc, x+len-1+LINEEND, y1);
	x += len;
	break;
/*...e*/
/*...sLST_VERT_UP:24:*/
case LST_VERT_UP:
	y2 = yoxBot[x].y; y1 = y2 + len;
	MoveToEx(hdc, x, CY_ACTION, NULL);
	LineTo(hdc, x, y1+1-LINEEND);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y1, NULL);
	LineTo(hdc, x, y2-LINEEND);
	x++;
	break;
/*...e*/
/*...sLST_VERT_DOWN:24:*/
case LST_VERT_DOWN:
	y2 = yoxBot[x].y; y1 = y2 + len;
	MoveToEx(hdc, x, CY_ACTION, NULL);
	LineTo(hdc, x, y1+1-LINEEND);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, y1, NULL);
	LineTo(hdc, x, y2-LINEEND);
	x++;
	break;
/*...e*/
/*...sLST_DIAG_UP:24:*/
case LST_DIAG_UP:
	for ( j = 0; j < len; j++ )
		Rectangle(hdc, x+j, yoxBot[x+j].y, x+j+1, CY_ACTION);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x+len-1+LINEEND, yoxBot[x+len-1].y-LINEEND);
	x += len;
	break;
/*...e*/
/*...sLST_DIAG_DOWN:24:*/
case LST_DIAG_DOWN:
	for ( j = 0; j < len; j++ )
		Rectangle(hdc, x+j, yoxBot[x+j].y, x+j+1, CY_ACTION);
	SelectObject(hdc, hpenEdge);
	MoveToEx(hdc, x, yoxBot[x].y, NULL);
	LineTo(hdc, x+len-1+LINEEND, yoxBot[x+len-1].y+LINEEND);
	x += len;
	break;
/*...e*/
		}
	SelectObject(hdc, hpenOld);
	SelectObject(hdc, hbrushOld);
	}
}
/*...e*/

	DeleteObject(hpen);
	DeleteObject(hbrush);
	DeleteObject(hpenEdge);

	SetWindowOrgEx(hdc, 0, -(CY_WINDOW-CY_STATUS), NULL);
	PaintStatusBar(hdc);

	SetWindowOrgEx(hdc, 0, 0, NULL);
	RefreshAreas(areaDraw, hdcBacking, hdc);

	EndPaint(hwnd, &ps);
	}
/*...e*/

long FAR PASCAL AlphaWindowProc(
	HWND hwnd,
	unsigned message,
	WPARAM wParam,
	LPARAM lParam
	)
	{
	switch ( message )
		{
/*...sWM_CREATE     \45\ create:16:*/
case WM_CREATE:

	hwndError = hwnd;

	/* Determine the system font character sizes, for scrolling steps */

	{
	HDC hdc = GetDC(hwnd);
	TEXTMETRIC tm;
	int xChar, yChar;
	GetTextMetrics(hdc, &tm);
	xChar = tm.tmAveCharWidth;
	yChar = tm.tmHeight + tm.tmExternalLeading;
	ReleaseDC(hwnd, hdc);
	}
	return 0;
/*...e*/
/*...sWM_PAINT      \45\ paint client:16:*/
case WM_PAINT:
	switch ( iMode )
		{
		case MODE_TITLE:
			PaintTitle(hwnd);
			break;
		case MODE_GAME:
		case MODE_DEMO:
			PaintGame(hwnd);
			break;
		default:
			{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			EndPaint(hwnd, &ps);
			break;
			}
		}
	return 0;
/*...e*/
/*...sWM_CLOSE      \45\ close:16:*/
case WM_CLOSE:
	bKeepGoing = FALSE;
	DestroyWindow(hwnd);
	return 0;
/*...e*/
/*...sWM_DESTROY    \45\ destroy:16:*/
case WM_DESTROY:
	PostQuitMessage(0);
	return 0;
/*...e*/
/*...sWM_KEYDOWN    \45\ look for virtual keys:16:*/
case WM_KEYDOWN:
/*...skeep track of which keys are depressed:24:*/
switch ( wParam )
	{
	case VK_LEFT:
		bLeftPressed = TRUE;
		break;
	case VK_RIGHT:
		bRightPressed = TRUE;
		break;
	case VK_UP:
		bUpPressed = TRUE;
		break;
	case VK_DOWN:
		bDownPressed = TRUE;
		break;
	case VK_SPACE:
		bSpacePressed = TRUE;
		break;
	}
/*...e*/
	switch ( iMode )
		{
/*...sMODE_TITLE \45\ if start key pressed\44\ start game:32:*/
case MODE_TITLE:
	if ( wParam == VK_SPACE )
		{
		iMode = MODE_GAME;
		bDead  = FALSE;
		iLives = 3;
		iCycle = 0;
		iScore = 0;
		SetLevel(0);
		BuildInitLand();
		iFuel = iFuelLast = MAX_FUEL;
		xShip = X_SHIP_INIT;
		yShip = Y_SHIP_INIT;
		iLaserHeat = 0;
		bLaserOn = FALSE;
		NoBaddys();
		ResetAreas();
		InvalidateRect(hwnd, NULL, TRUE);
		UpdateWindow(hwnd);
#ifdef USE_MMTIMER
		dwTimeLast = timeGetTime();
		dwTimeCycle = (DWORD) T_CYCLE_INIT;
#endif
		return 0L;
		}
	break;
/*...e*/
/*...sMODE_GAME  \45\ process keys as in game:32:*/
case MODE_GAME:
	switch ( wParam )
		{
		case VK_F2:
			if ( GetKeyState(VK_F3)&~1 )
				NextLevel();
#ifdef USE_MMTIMER
			if ( GetKeyState(VK_F5)&~1 )
				if ( dwTimeCycle < T_CYCLE_INIT * 2 )
					++dwTimeCycle;
			if ( GetKeyState(VK_F6)&~1 )
				if ( dwTimeCycle > T_CYCLE_MIN )
					--dwTimeCycle;
#endif
			return 0L;
		case VK_ESCAPE:
			iMode = MODE_TITLE;
			lCounter = 0;
			InvalidateRect(hwnd, NULL, TRUE);
			UpdateWindow(hwnd);
			return 0L;
		case VK_LEFT:
		case VK_RIGHT:
		case VK_UP:
		case VK_DOWN:
		case VK_SPACE:
			return 0L;
		}
	break;
/*...e*/
/*...sMODE_DEMO  \45\ if any key pressed\44\ goto title screen:32:*/
case MODE_DEMO:
	iMode = MODE_TITLE;
	lCounter = 0;
	InvalidateRect(hwnd, NULL, TRUE);
	UpdateWindow(hwnd);
	return 0L;
/*...e*/
		}
	break;
/*...e*/
/*...sWM_KEYUP      \45\ look for virtual keys:16:*/
case WM_KEYUP:
/*...skey track of which keys are depressed:24:*/
switch ( wParam )
	{
	case VK_LEFT:
		bLeftPressed = FALSE;
		return 0L;
	case VK_RIGHT:
		bRightPressed = FALSE;
		return 0L;
	case VK_UP:
		bUpPressed = FALSE;
		return 0L;
	case VK_DOWN:
		bDownPressed = FALSE;
		return 0L;
	case VK_SPACE:
		bSpacePressed = FALSE;
		return 0L;
	}
/*...e*/
	break;
/*...e*/
#ifdef USE_WM_TIMER
/*...sWM_TIMER:16:*/
case WM_TIMER:
	if ( iMode == MODE_TITLE )
		AnimateTitleScreen(hwnd);
	else
		OneCycle(hwnd);
	SetTimer(hwnd, 1, 10, (TIMERPROC) NULL);
	break;
/*...e*/
#endif
#ifdef USE_MMTIMER
/*...sWM_USER:16:*/
case WM_USER:
	if ( iMode == MODE_TITLE )
		{
		AnimateTitleScreen(hwnd);
		timeSetEvent(T_TITLE, 1, TimeCallback, (DWORD) hwnd,
			TIME_ONESHOT|TIME_CALLBACK_FUNCTION);
		}
	else
		{
		DWORD dwTime = timeGetTime();
		if ( dwTime - dwTimeLast >= dwTimeCycle )
			{
			DWORD dwTimeSleep;
			dwTimeLast = dwTime;
			OneCycle(hwnd);
			dwTimeSleep = dwTimeLast + dwTimeCycle - timeGetTime();
			if ( (signed long) dwTimeSleep < 1 )
				dwTimeSleep = 1;
			else if ( dwTimeSleep > dwTimeCycle )
				dwTimeSleep = dwTimeCycle;
			timeSetEvent(dwTimeSleep, 1, TimeCallback, (DWORD) hwnd,
				TIME_ONESHOT|TIME_CALLBACK_FUNCTION);
			}
		else
			timeSetEvent(1, 1, TimeCallback, (DWORD) hwnd,
				TIME_ONESHOT|TIME_CALLBACK_FUNCTION);
		}
	break;
/*...e*/
#endif
		}
	return DefWindowProc(hwnd, message, wParam, lParam);
	}
/*...e*/
/*...sWinMain:0:*/
int PASCAL WinMain(
	HINSTANCE hinstThis,		/* Handle of this instance           */
	HINSTANCE hinstPrev,		/* Handle of previous instance       */
	LPSTR lpszCmdLine,		/* Command line                      */
	int iCmdShow			/* Window open/iconic at start       */
	)
	{
	MSG msg;
	HWND hwnd;
	int cxScreen, cyScreen, cyCaption;
	int xWindow, yWindow, cxWindow = CX_WINDOW, cyWindow = CY_WINDOW;
	HDC hdcScreen;

	hinst = hinstThis;

	if ( !hinstPrev )
/*...sregister the class:16:*/
{
WNDCLASS wc;

wc.lpszClassName = appName;
wc.hInstance     = hinst;
wc.lpfnWndProc   = AlphaWindowProc;
wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
wc.hIcon         = LoadIcon(hinst, "Icon");
wc.lpszMenuName  = NULL;
wc.hbrBackground = GetStockObject(NULL_BRUSH);
wc.style         = CS_HREDRAW | CS_VREDRAW;
wc.cbClsExtra    = 0;
wc.cbWndExtra    = 0;
RegisterClass(&wc);
}
/*...e*/

	hbmDigits  = LoadBitmap(hinst, "BitmapDigits" );
	hbmScore   = LoadBitmap(hinst, "BitmapScore"  );
	hbmHiScore = LoadBitmap(hinst, "BitmapHiScore");
	hbmFuel    = LoadBitmap(hinst, "BitmapFuel"   );
	hbmLifeSym = LoadBitmap(hinst, "BitmapLifeSym");

	LoadSprites();

	hdcScreen  = GetDC((HWND) NULL);
	hdcBacking = CreateCompatibleDC(hdcScreen);
	hbmBacking = CreateCompatibleBitmap(hdcScreen, CX_ACTION, CY_ACTION);
	hbmBackingOld = SelectObject(hdcBacking, hbmBacking);
	ReleaseDC((HWND) NULL, hdcScreen);

	cxScreen  = GetSystemMetrics(SM_CXSCREEN);
	cyScreen  = GetSystemMetrics(SM_CYSCREEN);
	cyCaption = GetSystemMetrics(SM_CYCAPTION);

	/* As soon as the window gets created below, it gets its WM_CREATE */

	xWindow = (cxScreen - cxWindow) / 2;
	yWindow = (cyScreen - cyWindow) / 2;
					/* Make client area big enough for   */
					/* half of the 'page'                */

	hwndError = hwnd = CreateWindow(
		appName,		/* Window class name                 */
		appName,		/* Window title                      */
		WS_BORDER|WS_SYSMENU|WS_MINIMIZEBOX,
		xWindow - 1,
		yWindow - 1 - cyCaption,
		1 + cxWindow + 1,
		1 + cyCaption + cyWindow,
		NULL,			/* No parent for this window         */
		NULL,			/* Use the class menu (not our own)  */
		hinst,			/* Who created the window            */
		NULL);			/* No parameters to pass on          */

	hHourGlass = LoadCursor(NULL, IDC_WAIT);

	iMode = MODE_TITLE;

	ShowWindow(hwnd, iCmdShow);	/* Display window on screen          */
#ifdef NEVER
	UpdateWindow(hwnd);		/* Send paint message to its window  */
					/* procedure so that it can be drawn */
#endif

#ifdef USE_PEEKMSG
	while ( bKeepGoing )
		if ( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) )
			DispatchMessage(&msg);
		else if ( iMode == MODE_TITLE )
			AnimateTitleScreen(hwnd);
		else
			OneCycle(hwnd);
#endif
#ifdef USE_WM_TIMER
	/* 10ms is fastest WM_TIMER delivery rate allowed */
	SetTimer(hwnd, 1, 10, (TIMERPROC) NULL);
	while ( bKeepGoing && GetMessage(&msg, NULL, 0, 0) != 0 )
		DispatchMessage(&msg);
#endif
#ifdef USE_MMTIMER
	/* 1ms is fastest MM timer speed */
	{
	TIMECAPS tc;
	timeGetDevCaps(&tc, sizeof(tc));
	if ( timeBeginPeriod(tc.wPeriodMin) != TIMERR_NOERROR )
		{
		debug("Can't set up Multimedia timer");
		exit(1);
		}
	timeSetEvent(T_TITLE, 1, TimeCallback, (DWORD) hwnd,
		TIME_ONESHOT|TIME_CALLBACK_FUNCTION);
	while ( bKeepGoing && GetMessage(&msg, NULL, 0, 0) != 0 )
		DispatchMessage(&msg);
	timeEndPeriod(tc.wPeriodMin);
	}
#endif

	SelectObject(hdcBacking, hbmBackingOld);
	DeleteObject(hbmBacking);
	DeleteDC(hdcBacking);

	FreeSprites();

	DeleteObject(hbmDigits );
	DeleteObject(hbmScore  );
	DeleteObject(hbmHiScore);
	DeleteObject(hbmFuel   );
	DeleteObject(hbmLifeSym);

	return ( msg.wParam );
	}
/*...e*/
