/*
Assumptions:
initially working with 8KHz sound effects
2 seconds is enough for any effect we desire
usage: sound ofile.sb t_total wave_type {freq-vol}
*/

/*...sincludes:0:*/
#include <stdio.h>
#include <math.h>
#include <fcntl.h>
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>
#define PI 3.14159265358979323846
/*...e*/

#define	FREQ 22050
typedef signed char SAMPLE;
static SAMPLE samples[2*FREQ];

/*...swrite_file:0:*/
static void write_file(const char *fn, int len)
	{
	int fd;
	if ( (fd = open(fn, O_CREAT|O_TRUNC|O_WRONLY|O_BINARY, S_IREAD|S_IWRITE)) == -1 )
		{
		fprintf(stderr, "can't write %s", fn);
		exit(1);
		}
	write(fd, samples, len);
	close(fd);
	}
/*...e*/
/*...swave functions:0:*/
typedef double (*WAVE_FUNC)(int t, double freq);

static double sin_wave(int t, double freq)
	{
	return sin((double) t/(double)FREQ*2.0*PI * freq);
	}

static double squ_wave(int t, double freq)
	{
	if ( fmod( (double)t/(double)FREQ * freq, 1.0) <= 0.5 )
		return 1.0;
	else
		return -1.0;
	}

static double tri_wave(int t, double freq)
	{
	return fmod( (double)t/(double)FREQ * freq, 1.0) * 2.0 - 1.0;
	}
/*...e*/
/*...stabular sound generation:0:*/
typedef struct { int freq; double vol; } TABSND;

static void do_tabsnd(TABSND ts[], int elems, int t_total, WAVE_FUNC wf)
	{
	int i, j;
	for ( i = 0; i < elems; i++ )
		for ( j = 0; j < t_total/elems; j++ )
			samples[i*(t_total/elems)+j] = (SAMPLE)
				(ts[i].vol * 127.0 * (*wf)(j, ts[i].freq));
	}
/*...e*/
/*...szap:0:*/
static TABSND zap[] =
	{
	1000,	1.0,
	800,	0.9,
	600,	0.8,
	400,	0.7,
	200,	0.6,
	150,	0.3,
	100,	0.2,
	50,	0.1,
	};
/*...e*/
static int n_tabsnd = 0;
static TABSND tabsnd[100];

int main(int argc, char *argv[])
	{
	int i, t_total, wave_type;
	WAVE_FUNC wf;
#if 0
/*...ssome testcards:8:*/
/* First, some 'testcards' */
for ( i = 0; i < FREQ; i++ ) /* 1 second */
	samples[i] = (SAMPLE) ( 127.0 * sin_wave(i, 500.0) );
write_file("500hzsin.sb", FREQ);
for ( i = 0; i < FREQ; i++ ) /* 1 second */
	samples[i] = (SAMPLE) ( 127.0 * squ_wave(i, 500.0) );
write_file("500hzsqu.sb", FREQ);
for ( i = 0; i < FREQ; i++ ) /* 1 second */
	samples[i] = (SAMPLE) ( 127.0 * tri_wave(i, 500.0) );
write_file("500hztri.sb", FREQ);
/*...e*/
#endif
	if ( argc < 5 )
		return 1;

	sscanf(argv[2], "%i", &t_total);
	sscanf(argv[3], "%i", &wave_type);

	switch ( wave_type )
		{
		case 0:	wf = sin_wave; break;
		case 1:	wf = squ_wave; break;
		case 2:	wf = tri_wave; break;
		default: return 2;
		}

	for ( i = 1; i < argc; i++ )
		{
		int j, freq, cnt = 1;
		double vol;
		sscanf(argv[i], "%i-%lf-%i", &freq, &vol, &cnt);
		for ( j = 0; j < cnt; j++ )
			{
			tabsnd[n_tabsnd].freq = freq;
			tabsnd[n_tabsnd].vol  = vol ;
			n_tabsnd++;
			}
		}
	do_tabsnd(tabsnd, n_tabsnd, t_total, wf);
	write_file(argv[1], t_total);
	return 0;
	}
